@@ -16,8 +16,6 @@ set -ue
 TARGET="$(uname -m)"
 TARGET_CFLAGS=''
 QUIET='VERBOSE=1'
-CMAKE_BUILD_TYPE='RelWithDebInfo'
-DEBUG_COMMENT=''
 WITH_JEMALLOC=''
 DEBUG_EXTNAME=''
 WITH_SSL='/usr'
@@ -32,8 +30,11 @@ TAG=''
 CMAKE_BUILD_TYPE=''
 COMMON_FLAGS=''
 #
+TOKUDB_BACKUP_VERSION='@@TOKUDB_BACKUP_VERSION@@'
+#
 # Some programs that may be overriden
 TAR=${TAR:-tar}
+SCONS_ARGS=${SCONS_ARGS:-""}
 
 # Check if we have a functional getopt(1)
 if ! getopt --test
@@ -59,6 +60,7 @@ do
         CMAKE_BUILD_TYPE='Debug'
         BUILD_COMMENT="${BUILD_COMMENT:-}-debug"
         DEBUG_EXTNAME='-DDEBUG_EXTNAME=OFF'
+        SCONS_ARGS+=' debug=1'
         ;;
     -v | --valgrind )
         shift
@@ -183,17 +185,16 @@ PRODUCT="Percona-XtraDB-Cluster-$MYSQL_VERSION-$PERCONA_XTRADB_CLUSTER_VERSION"
 # Build information
 if test -e "$SOURCEDIR/Docs/INFO_SRC"
 then
-    REVISION="$(cd "$SOURCEDIR"; grep '^revno: ' Docs/INFO_SRC |sed -e 's/revno: //')"
-elif test -e "$SOURCEDIR/.bzr/branch/last-revision"
+    REVISION="$(cd "$SOURCEDIR"; grep '^short: ' Docs/INFO_SRC |sed -e 's/short: //')"
+elif [ -n "$(which git)" -a -d "$SOURCEDIR/.git" ];
 then
-    REVISION="$(cd "$SOURCEDIR"; cat .bzr/branch/last-revision | awk -F ' ' '{print $1}')"
+    REVISION="$(git rev-parse --short HEAD)"
 else
     REVISION=""
 fi
-REVISION="$(cd "$SOURCEDIR"; grep '^revno: ' Docs/INFO_SRC |sed -e 's/revno: //')"
 WSREP_VERSION="$(grep WSREP_INTERFACE_VERSION wsrep/wsrep_api.h | cut -d '"' -f2).$(grep 'SET(WSREP_PATCH_VERSION'  "cmake/wsrep.cmake" | cut -d '"' -f2)"
 GALERA_REVISION="$(cd "$SOURCEDIR/percona-xtradb-cluster-galera"; test -r GALERA-REVISION && cat GALERA-REVISION)"
-PRODUCT_FULL="$PRODUCT-$RELEASE_TAG$WSREP_VERSION.$REVISION${BUILD_COMMENT:-}$TAG.$(uname -s).$TARGET"
+PRODUCT_FULL="$PRODUCT-$RELEASE_TAG$WSREP_VERSION.${BUILD_COMMENT:-}$TAG.$(uname -s).$TARGET"
 COMMENT="Percona XtraDB Cluster binary (GPL) $MYSQL_VERSION-$RELEASE_TAG$WSREP_VERSION"
 COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}"
 
@@ -204,7 +205,7 @@ export CXX=${CXX:-g++}
 # TokuDB cmake flags
 if test -d "$SOURCEDIR/storage/tokudb"
 then
-    CMAKE_OPTS="${CMAKE_OPTS:-} -DBUILD_TESTING=OFF -DUSE_GTAGS=OFF -DUSE_CTAGS=OFF -DUSE_ETAGS=OFF -DUSE_CSCOPE=OFF"
+    CMAKE_OPTS="${CMAKE_OPTS:-} -DBUILD_TESTING=OFF -DUSE_GTAGS=OFF -DUSE_CTAGS=OFF -DUSE_ETAGS=OFF -DUSE_CSCOPE=OFF -DTOKUDB_BACKUP_PLUGIN_VERSION=${TOKUDB_BACKUP_VERSION}"
     
     if test "x$CMAKE_BUILD_TYPE" != "xDebug"
     then
@@ -224,6 +225,18 @@ if [ -n "$(which rpm)" ]; then
   export COMMON_FLAGS=$(rpm --eval %optflags | sed -e "s|march=i386|march=i686|g")
 else
   COMMON_FLAGS="-Wall -Wp,-D_FORTIFY_SOURCE=2 -DPERCONA_INNODB_VERSION=$MYSQL_RELEASE "
+  # Attempt to remove any optimisation flags from the debug build
+  # BLD-238 - bug1408232
+  if test "x$CMAKE_BUILD_TYPE" = "xDebug"
+  then
+    COMMON_FLAGS=`echo " ${COMMON_FLAGS} " | \
+              sed -e 's/ -O[0-9]* / /' \
+                  -e 's/-Wp,-D_FORTIFY_SOURCE=2/ /' \
+                  -e 's/ -unroll2 / /' \
+                  -e 's/ -ip / /' \
+                  -e 's/^ //' \
+                  -e 's/ $//'`
+  fi
 fi
 export CFLAGS=" $COMMON_FLAGS -static-libgcc $TARGET_CFLAGS ${CFLAGS:-}"
 export CXXFLAGS=" $COMMON_FLAGS $TARGET_CFLAGS ${CXXFLAGS:-}"
@@ -254,14 +267,14 @@ fi
         cd "percona-xtradb-cluster-galera"
         if grep builtin <<< "$STAG";then 
             # No builtin SSL in galera yet.
-            scons $MAKE_JFLAG --config=force ssl=0 revno="$GALERA_REVISION" boost_pool=0 \
+            scons $MAKE_JFLAG --config=force ssl=0 revno="$GALERA_REVISION" ${SCONS_ARGS} boost_pool=0 \
                 garb/garbd libgalera_smm.so
         elif grep static <<< "$STAG";then 
             # Disable SSL in galera for now
             scons $MAKE_JFLAG --config=force static_ssl=1 with_ssl=$GALERA_SSL \
-            revno="$GALERA_REVISION" boost_pool=0 garb/garbd libgalera_smm.so
+            revno="$GALERA_REVISION" ${SCONS_ARGS} boost_pool=0 garb/garbd libgalera_smm.so
         else 
-            scons $MAKE_JFLAG --config=force revno="$GALERA_REVISION" \
+            scons $MAKE_JFLAG --config=force revno="$GALERA_REVISION" ${SCONS_ARGS} \
                 garb/garbd libgalera_smm.so
         fi
         mkdir -p "$WORKDIR/usr/local/$PRODUCT_FULL/bin" \
@@ -291,7 +304,8 @@ fi
         -DMYSQL_SERVER_SUFFIX="-$RELEASE_TAG$WSREP_VERSION" \
         -DWITH_INNODB_DISALLOW_WRITES=ON \
         -DWITH_WSREP=ON \
-        -DMYSQL_MAINTAINER_MODE=OFF \
+        -DWITH_READLINE=system \
+        -DWITHOUT_TOKUDB=ON \
         -DCOMPILATION_COMMENT="$COMMENT" \
         -DWITH_PAM=ON \
         -DWITH_INNODB_MEMCACHED=ON \
@@ -308,13 +322,14 @@ fi
             -DWITH_EMBEDDED_SERVER=OFF \
             -DFEATURE_SET=community \
             -DENABLE_DTRACE=OFF \
-            -DMYSQL_MAINTAINER_MODE=OFF \
             $SSL_OPT \
             -DCMAKE_INSTALL_PREFIX="/usr/local/$PRODUCT_FULL" \
             -DMYSQL_DATADIR="/usr/local/$PRODUCT_FULL/data" \
             -DMYSQL_SERVER_SUFFIX="-$RELEASE_TAG$WSREP_VERSION" \
             -DWITH_INNODB_DISALLOW_WRITES=ON \
             -DWITH_WSREP=ON \
+            -DWITH_READLINE=system \
+            -DWITHOUT_TOKUDB=ON \
             -DCOMPILATION_COMMENT="$COMMENT - UNIV_DEBUG ON" \
             -DWITH_PAM=ON \
             -DWITH_INNODB_MEMCACHED=ON \
@@ -330,7 +345,7 @@ fi
        echo "Packaging the test files"
        # mkdir -p $WORKDIR/usr/local/$PRODUCT_FULL
        cp -R percona-xtradb-cluster-tests $WORKDIR/usr/local/$PRODUCT_FULL/
-    )
+    ) || exit 1
 
     # Build jemalloc
     if test "x$WITH_JEMALLOC" != x
@@ -346,25 +361,14 @@ fi
         # Copy COPYING file
         cp COPYING "$WORKDIR/usr/local/$PRODUCT_FULL/COPYING-jemalloc"
 
-    )
+    ) || exit 1
     fi
 
-)
+) || exit 1
 
 # Package the archive
 (
     cd "$WORKDIR/usr/local/"
 
-    find $PRODUCT_FULL ! -type d  ! \( -iname '*toku*' -o -iwholename '*/tokudb*/*'  \) | sort > $WORKDIR/tokudb_server.list
-    $TAR --owner=0 --group=0 -czf "$WORKDIR/$PRODUCT_FULL.tar.gz" -T $WORKDIR/tokudb_server.list
-    rm -f $WORKDIR/tokudb_server.list
-
-    if test -e "$PRODUCT_FULL/lib/mysql/plugin/ha_tokudb.so"
-    then
-        TARGETTOKU=$(echo $PRODUCT_FULL | sed 's/.Linux/.TokuDB.Linux/')
-	find $PRODUCT_FULL ! -type d \( -iname '*toku*' -o -iwholename '*/tokudb*/*' \) > $WORKDIR/tokudb_plugin.list
-        $TAR --owner=0 --group=0 -czf "$WORKDIR/$TARGETTOKU.tar.gz" -T $WORKDIR/tokudb_plugin.list
-        rm -f $WORKDIR/tokudb_plugin.list
-    fi
-)
-
+    $TAR --owner=0 --group=0 -czf "$WORKDIR/$PRODUCT_FULL.tar.gz" $PRODUCT_FULL 
+) || exit 1
