@@ -18,11 +18,10 @@ niceness=0
 mysqld_ld_preload=
 mysqld_ld_library_path=
 load_jemalloc=1
+load_hotbackup=0
 flush_caches=0
 numa_interleave=0
 resume_on_fail=1
-# Change (disable) transparent huge pages (TokuDB requirement)
-thp_setting=
 
 # Initial logging status: error log is not open, and not using syslog
 logging=init
@@ -102,7 +101,6 @@ Usage: $0 [OPTIONS]
                              starting the server
   --numa-interleave          Run mysqld with its memory interleaved
                              on all NUMA nodes
-${thp_usage}
 
 All other options are passed to the mysqld program.
 
@@ -345,6 +343,7 @@ parse_arguments() {
       --open_files_limit=*) open_files="$val" ;;
       --skip-kill-mysqld*) KILL_MYSQLD=0 ;;
       --thp-setting=*) thp_setting="$val" ;;
+      --preload-hotbackup) load_hotbackup=1 ;;
       --syslog) want_syslog=1 ;;
       --skip-syslog) want_syslog=0 ;;
       --syslog-tag=*) syslog_tag="$val" ;;
@@ -633,7 +632,7 @@ parse_arguments PICK-ARGS-FROM-ARGV "$@"
 #
 if test $load_jemalloc -eq 1
 then
-  for libjemall in "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
+  for libjemall in "${MY_BASEDIR_VERSION}/lib/mysql" "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
     if [ -r "$libjemall/libjemalloc.so.1" ]; then
       add_mysqld_ld_preload "$libjemall/libjemalloc.so.1"
       break
@@ -641,6 +640,19 @@ then
   done
 fi
 
+#
+# Add TokuDB HotBackup library to ld_preload
+#
+if test $load_hotbackup -eq 1
+then
+  for libhb in "${MY_BASEDIR_VERSION}/lib" "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
+    if [ -r "$libhb/libHotBackup.so" ]; then
+      add_mysqld_ld_preload "$libhb/libHotBackup.so"
+      break
+    fi
+  done
+fi
+
 #
 # Try to find the plugin directory
 #
@@ -650,7 +662,7 @@ if [ -n "${PLUGIN_DIR}" ]; then
   plugin_dir="${PLUGIN_DIR}"
 else
   # Try to find plugin dir relative to basedir
-  for dir in lib/mysql/plugin lib/plugin
+  for dir in lib64/mysql/plugin lib64/plugin lib/mysql/plugin lib/plugin
   do
     if [ -d "${MY_BASEDIR_VERSION}/${dir}" ]; then
       plugin_dir="${MY_BASEDIR_VERSION}/${dir}"
@@ -1034,9 +1046,6 @@ have_sleep=1
 # maximum number of wsrep restarts
 max_wsrep_restarts=0
 
-# maximum number of wsrep restarts
-max_wsrep_restarts=0
-
 if [ $wsrep_data_home_dir ];then 
     grastate_loc="$wsrep_data_home_dir/grastate.dat"
 else 
