@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -82,11 +82,12 @@
 #include "sql_acl.h"                       // SUPER_ACL
 #include <hash.h>
 #include <assert.h>
-
+#include "my_atomic.h"
 #ifdef WITH_WSREP
 #include "wsrep_mysqld.h"
 #endif /* WITH_WSREP */
 
+#include "my_atomic.h"
 /**
   @defgroup Locking Locking
   @{
@@ -120,7 +121,7 @@ static int
 lock_tables_check(THD *thd, TABLE **tables, uint count, uint flags)
 {
   uint system_count= 0, i= 0;
-  bool is_superuser= false;
+  bool enforce_ro= true;
   /*
     Identifies if the executed sql command can updated either a log
     or rpl info table.
@@ -129,7 +130,8 @@ lock_tables_check(THD *thd, TABLE **tables, uint count, uint flags)
 
   DBUG_ENTER("lock_tables_check");
 
-  is_superuser= thd->security_ctx->master_access & SUPER_ACL;
+  if (!opt_super_readonly)
+    enforce_ro= !(thd->security_ctx->master_access & SUPER_ACL);
   log_table_write_query=
      is_log_table_write_query(thd->lex->sql_command);
 
@@ -201,9 +203,10 @@ lock_tables_check(THD *thd, TABLE **tables, uint count, uint flags)
     if (!(flags & MYSQL_LOCK_IGNORE_GLOBAL_READ_ONLY) && !t->s->tmp_table)
     {
       if (t->reginfo.lock_type >= TL_WRITE_ALLOW_WRITE &&
-          !is_superuser && opt_readonly && !thd->slave_thread)
+          enforce_ro && opt_readonly && !thd->slave_thread)
       {
-        my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--read-only");
+        my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0),
+                 opt_super_readonly ? "--read-only (super)" : "--read-only");
         DBUG_RETURN(1);
       }
     }
@@ -911,6 +914,7 @@ static void print_lock_error(int error, const char *table)
   DBUG_VOID_RETURN;
 }
 
+volatile int32 Global_read_lock::m_active_requests;
 
 /****************************************************************************
   Handling of global read locks
@@ -1001,9 +1005,15 @@ bool Global_read_lock::lock_global_read_lock(THD *thd)
 
     mdl_request.init(MDL_key::GLOBAL, "", "", MDL_SHARED, MDL_EXPLICIT);
 
+    /* Increment static variable first to signal innodb memcached server
+       to release mdl locks held by it */
+    my_atomic_add32(&Global_read_lock::m_active_requests, 1);
     if (thd->mdl_context.acquire_lock(&mdl_request,
                                       thd->variables.lock_wait_timeout))
+    {
+      my_atomic_add32(&Global_read_lock::m_active_requests, -1);
       DBUG_RETURN(1);
+    }
 
     m_mdl_global_shared_lock= mdl_request.ticket;
     m_state= GRL_ACQUIRED;
@@ -1040,15 +1050,15 @@ void Global_read_lock::unlock_global_read_lock(THD *thd)
   {
     thd->mdl_context.release_lock(m_mdl_blocks_commits_lock);
     m_mdl_blocks_commits_lock= NULL;
-#ifdef WITH_WSREP
-    wsrep_locked_seqno= WSREP_SEQNO_UNDEFINED;
-    wsrep->resume(wsrep);
-#endif /* WITH_WSREP */
   }
+#ifdef WITH_WSREP
+  if (!provider_resumed())
+        wsrep_resume();
+#endif /* WITH_WSREP */
   thd->mdl_context.release_lock(m_mdl_global_shared_lock);
+  my_atomic_add32(&Global_read_lock::m_active_requests, -1);
   m_mdl_global_shared_lock= NULL;
   m_state= GRL_NONE;
- 
   DBUG_VOID_RETURN;
 }
 
@@ -1076,8 +1086,6 @@ bool Global_read_lock::make_global_read_lock_block_commit(THD *thd)
     If we didn't succeed lock_global_read_lock(), or if we already suceeded
     make_global_read_lock_block_commit(), do nothing.
   */
-  if (m_state != GRL_ACQUIRED)
-    DBUG_RETURN(0);
 
 #ifdef WITH_WSREP
   if (m_mdl_blocks_commits_lock)
@@ -1087,11 +1095,14 @@ bool Global_read_lock::make_global_read_lock_block_commit(THD *thd)
     thd->mdl_context.release_lock(m_mdl_blocks_commits_lock);
     m_mdl_blocks_commits_lock= NULL;
     wsrep_locked_seqno= WSREP_SEQNO_UNDEFINED;
-    wsrep->resume(wsrep);
+    DBUG_ASSERT(!provider_resumed());
+    wsrep_resume();
     m_state= GRL_ACQUIRED;
   }
 #endif /* WITH_WSREP */
 
+  if (m_state != GRL_ACQUIRED)
+    DBUG_RETURN(0);
 
   mdl_request.init(MDL_key::COMMIT, "", "", MDL_SHARED, MDL_EXPLICIT);
 
@@ -1103,24 +1114,79 @@ bool Global_read_lock::make_global_read_lock_block_commit(THD *thd)
   m_state= GRL_ACQUIRED_AND_BLOCKS_COMMIT;
 
 #ifdef WITH_WSREP
-  long long ret = wsrep->pause(wsrep);
+  if (!wsrep_pause())
+    DBUG_RETURN(TRUE);
+#endif
+  DBUG_RETURN(FALSE);
+}
+
+#ifdef WITH_WSREP
+/**
+  Pause the galera provider.
+  Also set wsrep_locked_seqno to sequence number returned.
+
+  @retval False  Failed to pause the provider, wsrep_locked_seqno is reset.
+  @retval True   Provider has been paused.
+*/
+bool Global_read_lock::wsrep_pause(void)
+{
+  wsrep_seqno_t ret = wsrep->pause(wsrep);
   if (ret >= 0)
   {
     wsrep_locked_seqno= ret;
+    pause_provider(TRUE);
   }
   else if (ret != -ENOSYS) /* -ENOSYS - no provider */
   {
-    WSREP_ERROR("Failed to pause provider: %lld (%s)", -ret, strerror(-ret));
+    WSREP_ERROR("Failed to pause provider: %lld (%s)", (long long)-ret, strerror(-ret));
 
     /* m_mdl_blocks_commits_lock is always NULL here */
     wsrep_locked_seqno= WSREP_SEQNO_UNDEFINED;
     my_error(ER_LOCK_DEADLOCK, MYF(0));
-    DBUG_RETURN(TRUE);
+    return FALSE;
   }
-#endif /* WITH_WSREP */
-  DBUG_RETURN(FALSE);
+  return TRUE;
 }
 
+/**
+  Pause the galera provider.
+  Also set wsrep_locked_seqno to sequence number returned.
+
+  @retval False  Failed to pause the provider, wsrep_locked_seqno is reset.
+  @retval True   Provider has been paused.
+*/
+wsrep_status_t Global_read_lock::wsrep_resume(void)
+{
+    wsrep_status_t ret(WSREP_WARNING);
+
+    wsrep_locked_seqno= WSREP_SEQNO_UNDEFINED;
+    ret = wsrep->resume(wsrep);
+
+    if (!ret)
+    {
+        pause_provider(FALSE);
+    }
+    else
+    {
+       WSREP_WARN("Failed to resume provider: %d", ret);
+    }
+    return ret;
+}
+
+bool Global_read_lock::wsrep_pause_once(void)
+{
+    if (!provider_paused)
+        return wsrep_pause();
+    return TRUE;
+}
+wsrep_status_t Global_read_lock::wsrep_resume_once(void)
+{
+    if (provider_paused)
+        return wsrep_resume();
+    return WSREP_OK;
+}
+
+#endif /* WITH_WSREP */
 
 /**
   Set explicit duration for metadata locks which are used to implement GRL.
@@ -1188,6 +1254,49 @@ void Global_backup_lock::release(THD *thd)
   DBUG_VOID_RETURN;
 }
 
+/**
+   Set explicit lock duration for the backup metadata lock.
+
+   @param thd     Reference to connection.
+*/
+
+void Global_backup_lock::set_explicit_locks_duration(THD *thd)
+{
+  bool should_own;
+
+  DBUG_ENTER("Global_backup_lock::set_explicit_lock_duration");
+
+  if (m_lock)
+  {
+    should_own= true;
+    thd->mdl_context.set_lock_duration(m_lock, MDL_EXPLICIT);
+  }
+  else
+  {
+    should_own= false;
+  }
+
+  DBUG_ASSERT(should_own ==
+              thd->mdl_context.is_lock_owner(m_namespace, "", "",
+                                             MDL_SHARED));
+
+  if (m_prot_lock)
+  {
+    should_own= true;
+    thd->mdl_context.set_lock_duration(m_prot_lock, MDL_EXPLICIT);
+  }
+  else
+  {
+    should_own= false;
+  }
+
+  DBUG_ASSERT(should_own ==
+              thd->mdl_context.is_lock_owner(m_namespace, "", "",
+                                             MDL_INTENTION_EXCLUSIVE));
+
+  DBUG_VOID_RETURN;
+}
+
 /**
    Acquire protection against a global backup lock. Wait if a global backup lock
    is active.
