@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
    Copyright (c) 2009, 2013, Monty Program Ab
    Copyright (C) 2012 Percona Inc.
 
@@ -1077,7 +1077,7 @@ bool LOGGER::slow_log_print(THD *thd, const char *query, uint query_length)
 
     /* fill in user_host value: the format is "%s[%s] @ %s [%s]" */
     user_host_len= (strxnmov(user_host_buff, MAX_USER_HOST_SIZE,
-                             sctx->priv_user ? sctx->priv_user : "", "[",
+                             sctx->priv_user, "[",
                              sctx->user ? sctx->user : (thd->slave_thread ?
                                                         "SQL_SLAVE" : ""),
                              "] @ ",
@@ -1178,12 +1178,6 @@ bool LOGGER::general_log_print(THD *thd, enum enum_server_command command,
   else
     message_buff[0]= '\0';
 
-  mysql_audit_general_log(thd, command, message_buff, message_buff_len);
-
-  /* Print the message to the buffer if we want to log this kind of commands */
-  if (! logger.log_command(thd, command))
-    return FALSE;
-
   return general_log_write(thd, command, message_buff, message_buff_len);
 }
 
@@ -1426,7 +1420,7 @@ static void setup_windows_event_source()
     nonzero if not possible to get unique filename.
 */
 
-static int find_uniq_filename(char *name, ulong *next)
+static int find_uniq_filename(char *name, ulong *next, bool need_next)
 {
   uint                  i;
   char                  buff[FN_REFLEN], ext_buf[FN_REFLEN];
@@ -1455,7 +1449,7 @@ static int find_uniq_filename(char *name, ulong *next)
   file_info= dir_info->dir_entry;
   for (i= dir_info->number_off_files ; i-- ; file_info++)
   {
-    if (memcmp(file_info->name, start, length) == 0 &&
+    if (strncmp(file_info->name, start, length) == 0 &&
 	test_if_number(file_info->name+length, &number,0))
     {
       set_if_bigger(max_found,(ulong) number);
@@ -1473,7 +1467,7 @@ updating the index files.", max_found);
     goto end;
   }
 
-  *next= max_found + 1;
+  *next= (need_next || max_found == 0) ? max_found + 1 : max_found;
   if (sprintf(ext_buf, "%06lu", *next) < 0)
   {
     error= 1;
@@ -1526,14 +1520,13 @@ void MYSQL_LOG::init(enum_log_type log_type_arg,
 bool MYSQL_LOG::init_and_set_log_file_name(const char *log_name,
                                            const char *new_name,
                                            enum_log_type log_type_arg,
-                                           enum cache_type io_cache_type_arg,
-                                           bool unique)
+                                           enum cache_type io_cache_type_arg)
 {
   init(log_type_arg, io_cache_type_arg);
 
   if (new_name && !strmov(log_file_name, new_name))
     return TRUE;
-  else if (!new_name && generate_new_name(log_file_name, log_name, unique))
+  else if (!new_name && generate_new_name(log_file_name, log_name))
     return TRUE;
 
   return FALSE;
@@ -1566,8 +1559,7 @@ bool MYSQL_LOG::open(
                      PSI_file_key log_file_key,
 #endif
                      const char *log_name, enum_log_type log_type_arg,
-                     const char *new_name, enum cache_type io_cache_type_arg,
-                     bool unique)
+                     const char *new_name, enum cache_type io_cache_type_arg)
 {
   char buff[FN_REFLEN];
   MY_STAT f_stat;
@@ -1586,7 +1578,7 @@ bool MYSQL_LOG::open(
   }
 
   if (init_and_set_log_file_name(name, new_name,
-                                 log_type_arg, io_cache_type_arg, unique) ||
+                                 log_type_arg, io_cache_type_arg) ||
       DBUG_EVALUATE_IF("fault_injection_init_name", log_type == LOG_BIN, 0))
     goto err;
 
@@ -1664,6 +1656,8 @@ err:
     my_error(ER_BINLOG_LOGGING_IMPOSSIBLE, MYF(0), "Either disk is full or "
              "file system is read only while opening the binlog. Aborting the "
              "server");
+    sql_print_error("Either disk is full or file system is read only while "
+                    "opening the binlog. Aborting the server");
     thd->protocol->end_statement();
     _exit(EXIT_FAILURE);
   }
@@ -1767,15 +1761,14 @@ void MYSQL_LOG::cleanup()
 }
 
 
-int MYSQL_LOG::generate_new_name(char *new_name, const char *log_name,
-                                 bool unique)
+int MYSQL_LOG::generate_new_name(char *new_name, const char *log_name)
 {
   fn_format(new_name, log_name, mysql_data_home, "", 4);
-  if (unique)
+  if (log_type == LOG_BIN || max_slowlog_size > 0)
   {
     if (!fn_ext(log_name)[0])
     {
-      if (find_uniq_filename(new_name, &cur_log_ext))
+      if (find_uniq_filename(new_name, &cur_log_ext, log_type == LOG_BIN))
       {
         my_printf_error(ER_NO_UNIQUE_LOGFILE, ER(ER_NO_UNIQUE_LOGFILE),
                         MYF(ME_FATALERROR), log_name);
@@ -1846,7 +1839,7 @@ void MYSQL_QUERY_LOG::reopen_file()
 #ifdef HAVE_PSI_INTERFACE
        m_log_file_key,
 #endif
-       save_name, log_type, 0, io_cache_type, false);
+       save_name, log_type, 0, io_cache_type);
   my_free(save_name);
 
   mysql_mutex_unlock(&LOCK_log);
@@ -2014,6 +2007,9 @@ bool MYSQL_QUERY_LOG::write(THD *thd, ulonglong current_utime,
     char affected_row_buff[21];
     end= buff;
 
+    if (max_slowlog_size > 0)
+      error= rotate(max_slowlog_size, &need_purge);
+
     if (!(specialflag & SPECIAL_SHORT_LOG_FORMAT))
     {
       if (opt_slow_query_log_timestamp_always || current_time != last_time)
@@ -2089,7 +2085,7 @@ bool MYSQL_QUERY_LOG::write(THD *thd, ulonglong current_utime,
     if (my_b_write(&log_file, (uchar*) "\n", 1))
         tmp_errno= errno;
 
-    if (opt_log_slow_sp_statements &&
+    if (opt_log_slow_sp_statements == 1 &&
         thd->sp_runtime_ctx &&
         my_b_printf(&log_file,
                     "# Stored_routine: %s\n",
@@ -2219,8 +2215,6 @@ bool MYSQL_QUERY_LOG::write(THD *thd, ulonglong current_utime,
                         my_strerror(errbuf, sizeof(errbuf), errno));
       }
     }
-    if (max_slowlog_size > 0)
-      error= rotate(max_slowlog_size, &need_purge);
   }
   ulong save_cur_ext = cur_log_ext;
   mysql_mutex_unlock(&LOCK_log);
@@ -2266,7 +2260,7 @@ int MYSQL_QUERY_LOG::new_file()
   if (cur_log_ext == (ulong)-1)
   {
     strcpy(new_name, name);
-    if ((error= generate_new_name(new_name, name, true)))
+    if ((error= generate_new_name(new_name, name)))
       goto end;
   }
   else
@@ -2294,7 +2288,7 @@ int MYSQL_QUERY_LOG::new_file()
                 key_file_query_log,
 #endif
                 name,
-                LOG_NORMAL, new_name, WRITE_CACHE, false);
+                LOG_NORMAL, new_name, WRITE_CACHE);
 
   my_free(old_name);
 
@@ -2354,7 +2348,8 @@ bool slow_log_print(THD *thd, const char *query, uint query_length)
 }
 
 
-bool LOGGER::log_command(THD *thd, enum enum_server_command command)
+bool LOGGER::log_command(THD *thd, enum enum_server_command command,
+                         const char *query_str, size_t query_length)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   Security_context *sctx= thd->security_ctx;
@@ -2363,6 +2358,12 @@ bool LOGGER::log_command(THD *thd, enum enum_server_command command)
     Log command if we have at least one log event handler enabled and want
     to log this king of commands
   */
+
+  /* Audit notification when no general log handler present */
+  mysql_audit_general_log(thd, command_name[(uint) command].str,
+                          command_name[(uint) command].length,
+                          query_str, query_length);
+
   if (*general_log_handler_list && (what_to_log & (1L << (uint) command)))
   {
     if ((thd->variables.option_bits & OPTION_LOG_OFF)
@@ -2388,6 +2389,10 @@ bool general_log_print(THD *thd, enum enum_server_command command,
   va_list args;
   uint error= 0;
 
+  /* Print the message to the buffer if we want to log this king of commands */
+  if (! logger.log_command(thd, command, "", 0))
+    return FALSE;
+
   va_start(args, format);
   error= logger.general_log_print(thd, command, format, args);
   va_end(args);
@@ -2398,10 +2403,8 @@ bool general_log_print(THD *thd, enum enum_server_command command,
 bool general_log_write(THD *thd, enum enum_server_command command,
                        const char *query, uint query_length)
 {
-  mysql_audit_general_log(thd, command, query, query_length);
-
   /* Write the message to the log if we want to log this king of commands */
-  if (logger.log_command(thd, command))
+  if (logger.log_command(thd, command, query, query_length))
     return logger.general_log_write(thd, command, query, query_length);
 
   return FALSE;
@@ -2698,7 +2701,6 @@ int my_plugin_log_message(MYSQL_PLUGIN *plugin_ptr, plugin_log_level level,
 
 /********* transaction coordinator log for 2pc - mmap() based solution *******/
 
-
 /*
   the log consists of a file, mmapped to a memory.
   file is divided on pages of tc_log_page_size size.
