@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -286,6 +286,12 @@ static bool recreate_temporary_table(THD *thd, TABLE *table)
 
   table->file->info(HA_STATUS_AUTO | HA_STATUS_NO_LOCK);
 
+  /*
+    If LOCK TABLES list is not empty and contains this table
+    then unlock the table and remove it from this list.
+  */
+  mysql_lock_remove(thd, thd->lock, table);
+
   /* Don't free share. */
   close_temporary_table(thd, table, FALSE, FALSE);
 
@@ -430,6 +436,13 @@ bool Sql_cmd_truncate_table::truncate_table(THD *thd, TABLE_LIST *table_ref)
   if (is_temporary_table(table_ref))
   {
     TABLE *tmp_table= table_ref->table;
+    /*    
+      bug 1313901 : THD::decide_logging_format has not yet been called and
+                    may not be called at all dependig on the engine, so call it
+                    here.
+    */
+    if (thd->decide_logging_format(table_ref) != 0)
+      DBUG_RETURN(TRUE);
 
     /* In RBR, the statement is not binlogged if the table is temporary. */
     binlog_stmt= !thd->is_current_stmt_binlog_format_row();
