@@ -603,75 +603,76 @@
      TO_SYM = 821,
      TOKU_UNCOMPRESSED_SYM = 822,
      TOKU_ZLIB_SYM = 823,
-     TOKU_QUICKLZ_SYM = 824,
-     TOKU_LZMA_SYM = 825,
-     TOKU_FAST_SYM = 826,
-     TOKU_SMALL_SYM = 827,
-     TRAILING = 828,
-     TRANSACTION_SYM = 829,
-     TRIGGERS_SYM = 830,
-     TRIGGER_SYM = 831,
-     TRIM = 832,
-     TRUE_SYM = 833,
-     TRUNCATE_SYM = 834,
-     TYPES_SYM = 835,
-     TYPE_SYM = 836,
-     UDF_RETURNS_SYM = 837,
-     ULONGLONG_NUM = 838,
-     UNCOMMITTED_SYM = 839,
-     UNDEFINED_SYM = 840,
-     UNDERSCORE_CHARSET = 841,
-     UNDOFILE_SYM = 842,
-     UNDO_BUFFER_SIZE_SYM = 843,
-     UNDO_SYM = 844,
-     UNICODE_SYM = 845,
-     UNINSTALL_SYM = 846,
-     UNION_SYM = 847,
-     UNIQUE_SYM = 848,
-     UNKNOWN_SYM = 849,
-     UNLOCK_SYM = 850,
-     UNSIGNED = 851,
-     UNTIL_SYM = 852,
-     UPDATE_SYM = 853,
-     UPGRADE_SYM = 854,
-     USAGE = 855,
-     USER = 856,
-     USER_STATS_SYM = 857,
-     USE_FRM = 858,
-     USE_SYM = 859,
-     USING = 860,
-     UTC_DATE_SYM = 861,
-     UTC_TIMESTAMP_SYM = 862,
-     UTC_TIME_SYM = 863,
-     VALUES = 864,
-     VALUE_SYM = 865,
-     VARBINARY = 866,
-     VARCHAR = 867,
-     VARIABLES = 868,
-     VARIANCE_SYM = 869,
-     VARYING = 870,
-     VAR_SAMP_SYM = 871,
-     VIEW_SYM = 872,
-     WAIT_SYM = 873,
-     WARNINGS = 874,
-     WEEK_SYM = 875,
-     WEIGHT_STRING_SYM = 876,
-     WHEN_SYM = 877,
-     WHERE = 878,
-     WHILE_SYM = 879,
-     WITH = 880,
-     WITH_CUBE_SYM = 881,
-     WITH_ROLLUP_SYM = 882,
-     WORK_SYM = 883,
-     WRAPPER_SYM = 884,
-     WRITE_SYM = 885,
-     X509_SYM = 886,
-     XA_SYM = 887,
-     XML_SYM = 888,
-     XOR = 889,
-     YEAR_MONTH_SYM = 890,
-     YEAR_SYM = 891,
-     ZEROFILL = 892
+     TOKU_SNAPPY_SYM = 824,
+     TOKU_QUICKLZ_SYM = 825,
+     TOKU_LZMA_SYM = 826,
+     TOKU_FAST_SYM = 827,
+     TOKU_SMALL_SYM = 828,
+     TRAILING = 829,
+     TRANSACTION_SYM = 830,
+     TRIGGERS_SYM = 831,
+     TRIGGER_SYM = 832,
+     TRIM = 833,
+     TRUE_SYM = 834,
+     TRUNCATE_SYM = 835,
+     TYPES_SYM = 836,
+     TYPE_SYM = 837,
+     UDF_RETURNS_SYM = 838,
+     ULONGLONG_NUM = 839,
+     UNCOMMITTED_SYM = 840,
+     UNDEFINED_SYM = 841,
+     UNDERSCORE_CHARSET = 842,
+     UNDOFILE_SYM = 843,
+     UNDO_BUFFER_SIZE_SYM = 844,
+     UNDO_SYM = 845,
+     UNICODE_SYM = 846,
+     UNINSTALL_SYM = 847,
+     UNION_SYM = 848,
+     UNIQUE_SYM = 849,
+     UNKNOWN_SYM = 850,
+     UNLOCK_SYM = 851,
+     UNSIGNED = 852,
+     UNTIL_SYM = 853,
+     UPDATE_SYM = 854,
+     UPGRADE_SYM = 855,
+     USAGE = 856,
+     USER = 857,
+     USER_STATS_SYM = 858,
+     USE_FRM = 859,
+     USE_SYM = 860,
+     USING = 861,
+     UTC_DATE_SYM = 862,
+     UTC_TIMESTAMP_SYM = 863,
+     UTC_TIME_SYM = 864,
+     VALUES = 865,
+     VALUE_SYM = 866,
+     VARBINARY = 867,
+     VARCHAR = 868,
+     VARIABLES = 869,
+     VARIANCE_SYM = 870,
+     VARYING = 871,
+     VAR_SAMP_SYM = 872,
+     VIEW_SYM = 873,
+     WAIT_SYM = 874,
+     WARNINGS = 875,
+     WEEK_SYM = 876,
+     WEIGHT_STRING_SYM = 877,
+     WHEN_SYM = 878,
+     WHERE = 879,
+     WHILE_SYM = 880,
+     WITH = 881,
+     WITH_CUBE_SYM = 882,
+     WITH_ROLLUP_SYM = 883,
+     WORK_SYM = 884,
+     WRAPPER_SYM = 885,
+     WRITE_SYM = 886,
+     X509_SYM = 887,
+     XA_SYM = 888,
+     XML_SYM = 889,
+     XOR = 890,
+     YEAR_MONTH_SYM = 891,
+     YEAR_SYM = 892,
+     ZEROFILL = 893
    };
 #endif
 /* Tokens.  */
@@ -1241,75 +1242,76 @@
 #define TO_SYM 821
 #define TOKU_UNCOMPRESSED_SYM 822
 #define TOKU_ZLIB_SYM 823
-#define TOKU_QUICKLZ_SYM 824
-#define TOKU_LZMA_SYM 825
-#define TOKU_FAST_SYM 826
-#define TOKU_SMALL_SYM 827
-#define TRAILING 828
-#define TRANSACTION_SYM 829
-#define TRIGGERS_SYM 830
-#define TRIGGER_SYM 831
-#define TRIM 832
-#define TRUE_SYM 833
-#define TRUNCATE_SYM 834
-#define TYPES_SYM 835
-#define TYPE_SYM 836
-#define UDF_RETURNS_SYM 837
-#define ULONGLONG_NUM 838
-#define UNCOMMITTED_SYM 839
-#define UNDEFINED_SYM 840
-#define UNDERSCORE_CHARSET 841
-#define UNDOFILE_SYM 842
-#define UNDO_BUFFER_SIZE_SYM 843
-#define UNDO_SYM 844
-#define UNICODE_SYM 845
-#define UNINSTALL_SYM 846
-#define UNION_SYM 847
-#define UNIQUE_SYM 848
-#define UNKNOWN_SYM 849
-#define UNLOCK_SYM 850
-#define UNSIGNED 851
-#define UNTIL_SYM 852
-#define UPDATE_SYM 853
-#define UPGRADE_SYM 854
-#define USAGE 855
-#define USER 856
-#define USER_STATS_SYM 857
-#define USE_FRM 858
-#define USE_SYM 859
-#define USING 860
-#define UTC_DATE_SYM 861
-#define UTC_TIMESTAMP_SYM 862
-#define UTC_TIME_SYM 863
-#define VALUES 864
-#define VALUE_SYM 865
-#define VARBINARY 866
-#define VARCHAR 867
-#define VARIABLES 868
-#define VARIANCE_SYM 869
-#define VARYING 870
-#define VAR_SAMP_SYM 871
-#define VIEW_SYM 872
-#define WAIT_SYM 873
-#define WARNINGS 874
-#define WEEK_SYM 875
-#define WEIGHT_STRING_SYM 876
-#define WHEN_SYM 877
-#define WHERE 878
-#define WHILE_SYM 879
-#define WITH 880
-#define WITH_CUBE_SYM 881
-#define WITH_ROLLUP_SYM 882
-#define WORK_SYM 883
-#define WRAPPER_SYM 884
-#define WRITE_SYM 885
-#define X509_SYM 886
-#define XA_SYM 887
-#define XML_SYM 888
-#define XOR 889
-#define YEAR_MONTH_SYM 890
-#define YEAR_SYM 891
-#define ZEROFILL 892
+#define TOKU_SNAPPY_SYM 824
+#define TOKU_QUICKLZ_SYM 825
+#define TOKU_LZMA_SYM 826
+#define TOKU_FAST_SYM 827
+#define TOKU_SMALL_SYM 828
+#define TRAILING 829
+#define TRANSACTION_SYM 830
+#define TRIGGERS_SYM 831
+#define TRIGGER_SYM 832
+#define TRIM 833
+#define TRUE_SYM 834
+#define TRUNCATE_SYM 835
+#define TYPES_SYM 836
+#define TYPE_SYM 837
+#define UDF_RETURNS_SYM 838
+#define ULONGLONG_NUM 839
+#define UNCOMMITTED_SYM 840
+#define UNDEFINED_SYM 841
+#define UNDERSCORE_CHARSET 842
+#define UNDOFILE_SYM 843
+#define UNDO_BUFFER_SIZE_SYM 844
+#define UNDO_SYM 845
+#define UNICODE_SYM 846
+#define UNINSTALL_SYM 847
+#define UNION_SYM 848
+#define UNIQUE_SYM 849
+#define UNKNOWN_SYM 850
+#define UNLOCK_SYM 851
+#define UNSIGNED 852
+#define UNTIL_SYM 853
+#define UPDATE_SYM 854
+#define UPGRADE_SYM 855
+#define USAGE 856
+#define USER 857
+#define USER_STATS_SYM 858
+#define USE_FRM 859
+#define USE_SYM 860
+#define USING 861
+#define UTC_DATE_SYM 862
+#define UTC_TIMESTAMP_SYM 863
+#define UTC_TIME_SYM 864
+#define VALUES 865
+#define VALUE_SYM 866
+#define VARBINARY 867
+#define VARCHAR 868
+#define VARIABLES 869
+#define VARIANCE_SYM 870
+#define VARYING 871
+#define VAR_SAMP_SYM 872
+#define VIEW_SYM 873
+#define WAIT_SYM 874
+#define WARNINGS 875
+#define WEEK_SYM 876
+#define WEIGHT_STRING_SYM 877
+#define WHEN_SYM 878
+#define WHERE 879
+#define WHILE_SYM 880
+#define WITH 881
+#define WITH_CUBE_SYM 882
+#define WITH_ROLLUP_SYM 883
+#define WORK_SYM 884
+#define WRAPPER_SYM 885
+#define WRITE_SYM 886
+#define X509_SYM 887
+#define XA_SYM 888
+#define XML_SYM 889
+#define XOR 890
+#define YEAR_MONTH_SYM 891
+#define YEAR_SYM 892
+#define ZEROFILL 893
 
 
 
@@ -1319,7 +1321,7 @@ typedef union YYSTYPE
 {
 
 /* Line 2068 of yacc.c  */
-#line 967 "/mnt/workspace/percona-xtradb-cluster-5.6-source-tarball-bzr/sql/sql_yacc.yy"
+#line 968 "/mnt/workspace/percona-xtradb-cluster-5.6-source-tarball/sql/sql_yacc.yy"
 
   int  num;
   ulong ulong_num;
@@ -1380,7 +1382,7 @@ typedef union YYSTYPE
 
 
 /* Line 2068 of yacc.c  */
-#line 1384 "/mnt/workspace/percona-xtradb-cluster-5.6-source-tarball-bzr/sql/sql_yacc.h"
+#line 1386 "/mnt/workspace/percona-xtradb-cluster-5.6-source-tarball/sql/sql_yacc.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
