@@ -180,11 +180,13 @@ bool trans_begin(THD *thd, uint flags)
       Implicitly starting a RW transaction is allowed for backward
       compatibility.
     */
-    const bool user_is_super=
-      MY_TEST(thd->security_ctx->master_access & SUPER_ACL);
-    if (opt_readonly && !user_is_super)
+    bool enforce_ro= true;
+    if (!opt_super_readonly)
+      enforce_ro= !(thd->security_ctx->master_access & SUPER_ACL);
+    if (opt_readonly && enforce_ro)
     {
-      my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--read-only");
+      my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0),
+               opt_super_readonly ? "--read-only (super)" : "--read-only");
       DBUG_RETURN(true);
     }
     thd->tx_read_only= false;
@@ -580,6 +582,20 @@ bool trans_savepoint(THD *thd, LEX_STRING name)
       !opt_using_transactions)
     DBUG_RETURN(FALSE);
 
+  if (unlikely(thd->in_sub_stmt & SUB_STMT_TRIGGER))
+  {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "savepoints in triggers");
+    DBUG_RETURN(TRUE);
+  }
+
+  if (unlikely(thd->in_sub_stmt & SUB_STMT_FUNCTION))
+  {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "savepoints in functions");
+    DBUG_RETURN(TRUE);
+  }
+
+  DBUG_ASSERT(!thd->in_sub_stmt);
+
   enum xa_states xa_state= thd->transaction.xid_state.xa_state;
   if (xa_state != XA_NOTR && xa_state != XA_ACTIVE)
   {
@@ -664,6 +680,20 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_STRING name)
   thd->transaction.all.dbug_unsafe_rollback_flags("all");
 #endif
 
+  if (unlikely(thd->in_sub_stmt & SUB_STMT_TRIGGER))
+  {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "savepoints in triggers");
+    DBUG_RETURN(TRUE);
+  }
+
+  if (unlikely(thd->in_sub_stmt & SUB_STMT_FUNCTION))
+  {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "savepoints in functions");
+    DBUG_RETURN(TRUE);
+  }
+
+  DBUG_ASSERT(!thd->in_sub_stmt);
+
   if (sv == NULL)
   {
     my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "SAVEPOINT", name.str);
