@@ -1,4 +1,4 @@
-/* Copyright 2008-2013 Codership Oy <http://www.codership.com>
+/* Copyright 2008-2015 Codership Oy <http://www.codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 #include <mysqld.h>
 #include <sql_class.h>
 #include <sql_parse.h>
+#include "debug_sync.h"
 #include "wsrep_priv.h"
 #include "wsrep_thd.h"
 #include "wsrep_sst.h"
@@ -23,6 +24,7 @@
 #include "wsrep_var.h"
 #include "wsrep_binlog.h"
 #include "wsrep_applier.h"
+#include "wsrep_xid.h"
 #include <cstdio>
 #include <cstdlib>
 #include "log_event.h"
@@ -56,7 +58,6 @@ my_bool wsrep_certify_nonPK            = 1; // certify, even when no primary key
 long    wsrep_max_protocol_version     = 3; // maximum protocol version to use
 ulong   wsrep_forced_binlog_format     = BINLOG_FORMAT_UNSPEC;
 my_bool wsrep_recovery                 = 0; // recovery
-my_bool wsrep_replicate_myisam         = 0; // enable myisam replication
 my_bool wsrep_log_conflicts            = 0;
 ulong   wsrep_mysql_replication_bundle = 0;
 my_bool wsrep_desync                   = 0; // desynchronize the node from the
@@ -151,66 +152,24 @@ static void wsrep_log_states (wsrep_log_level_t   const level,
   wsrep_log_cb (level, msg);
 }
 
-static my_bool set_SE_checkpoint(THD* unused, plugin_ref plugin, void* arg)
-{
-  XID* xid= reinterpret_cast<XID*>(arg);
-  handlerton* hton= plugin_data(plugin, handlerton *);
-  if (hton->db_type == DB_TYPE_INNODB)
-  {
-    const wsrep_uuid_t* uuid(wsrep_xid_uuid(xid));
-    char uuid_str[40] = {0, };
-    wsrep_uuid_print(uuid, uuid_str, sizeof(uuid_str));
-    WSREP_DEBUG("Set WSREPXid for InnoDB:  %s:%lld",
-                uuid_str, (long long)wsrep_xid_seqno(xid));
-    hton->wsrep_set_checkpoint(hton, xid);
-  }
-  return FALSE;
-}
-
-void wsrep_set_SE_checkpoint(XID* xid)
-{
-  plugin_foreach(NULL, set_SE_checkpoint, MYSQL_STORAGE_ENGINE_PLUGIN, xid);
-}
-
-static my_bool get_SE_checkpoint(THD* unused, plugin_ref plugin, void* arg)
-{
-  XID* xid= reinterpret_cast<XID*>(arg);
-  handlerton* hton= plugin_data(plugin, handlerton *);
-  if (hton->db_type == DB_TYPE_INNODB)
-  {
-    hton->wsrep_get_checkpoint(hton, xid);
-    const wsrep_uuid_t* uuid(wsrep_xid_uuid(xid));
-    char uuid_str[40] = {0, };
-    wsrep_uuid_print(uuid, uuid_str, sizeof(uuid_str));
-    WSREP_DEBUG("Read WSREPXid from InnoDB:  %s:%lld",
-                uuid_str, (long long)wsrep_xid_seqno(xid));
-
-  }
-  return FALSE;
-}
-
-void wsrep_get_SE_checkpoint(XID* xid)
-{
-  plugin_foreach(NULL, get_SE_checkpoint, MYSQL_STORAGE_ENGINE_PLUGIN, xid);
-}
-
-void wsrep_init_sidno(const wsrep_uuid_t& uuid)
+void wsrep_init_sidno(const wsrep_uuid_t& wsrep_uuid)
 {
   /* generate new Sid map entry from inverted uuid */
   rpl_sid sid;
   wsrep_uuid_t ltid_uuid;
+
   for (size_t i= 0; i < sizeof(ltid_uuid.data); ++i)
   {
-      ltid_uuid.data[i] = ~local_uuid.data[i];
+      ltid_uuid.data[i] = ~wsrep_uuid.data[i];
   }
+
   sid.copy_from(ltid_uuid.data);
   global_sid_lock->wrlock();
   wsrep_sidno= global_sid_map->add_sid(sid);
-  WSREP_INFO("inited wsrep sidno %d", wsrep_sidno);
+  WSREP_INFO("Initialized wsrep sidno %d", wsrep_sidno);
   global_sid_lock->unlock();
 }
 
-
 static wsrep_cb_status_t
 wsrep_view_handler_cb (void*                    app_ctx,
                        void*                    recv_ctx,
@@ -345,11 +304,9 @@ wsrep_view_handler_cb (void*                    app_ctx,
         local_seqno= view->state_id.seqno;
       }
       /* Init storage engine XIDs from first view */
-      XID xid;
-      wsrep_xid_init(&xid, &local_uuid, local_seqno);
-      wsrep_set_SE_checkpoint(&xid);
-      new_status= WSREP_MEMBER_JOINED;
+      wsrep_set_SE_checkpoint(local_uuid, local_seqno);
       wsrep_init_sidno(local_uuid);
+      new_status= WSREP_MEMBER_JOINED;
     }
 
     // just some sanity check
@@ -457,38 +414,28 @@ static void wsrep_synced_cb(void* app_ctx)
 static void wsrep_init_position()
 {
   /* read XIDs from storage engines */
-  XID xid;
-  memset(&xid, 0, sizeof(xid));
-  xid.formatID= -1;
-  wsrep_get_SE_checkpoint(&xid);
+  wsrep_uuid_t uuid;
+  wsrep_seqno_t seqno;
+  wsrep_get_SE_checkpoint(uuid, seqno);
 
-  if (xid.formatID == -1)
+  if (!memcmp(&uuid, &WSREP_UUID_UNDEFINED, sizeof(wsrep_uuid_t)))
   {
     WSREP_INFO("Read nil XID from storage engines, skipping position init");
     return;
   }
-  else if (!wsrep_is_wsrep_xid(&xid))
-  {
-    WSREP_WARN("Read non-wsrep XID from storage engines, skipping position init");
-    return;
-  }
-
-  const wsrep_uuid_t* uuid= wsrep_xid_uuid(&xid);
-  const wsrep_seqno_t seqno= wsrep_xid_seqno(&xid);
 
   char uuid_str[40] = {0, };
-  wsrep_uuid_print(uuid, uuid_str, sizeof(uuid_str));
+  wsrep_uuid_print(&uuid, uuid_str, sizeof(uuid_str));
   WSREP_INFO("Initial position: %s:%lld", uuid_str, (long long)seqno);
 
-
   if (!memcmp(&local_uuid, &WSREP_UUID_UNDEFINED, sizeof(local_uuid)) &&
       local_seqno == WSREP_SEQNO_UNDEFINED)
   {
     // Initial state
-    local_uuid= *uuid;
+    local_uuid= uuid;
     local_seqno= seqno;
   }
-  else if (memcmp(&local_uuid, uuid, sizeof(local_uuid)) ||
+  else if (memcmp(&local_uuid, &uuid, sizeof(local_uuid)) ||
            local_seqno != seqno)
   {
     WSREP_WARN("Initial position was provided by configuration or SST, "
@@ -734,14 +681,12 @@ void wsrep_recover()
                uuid_str, (long long)local_seqno);
     return;
   }
-  XID xid;
-  memset(&xid, 0, sizeof(xid));
-  xid.formatID= -1;
-  wsrep_get_SE_checkpoint(&xid);
+  wsrep_uuid_t uuid;
+  wsrep_seqno_t seqno;
+  wsrep_get_SE_checkpoint(uuid, seqno);
   char uuid_str[40];
-  wsrep_uuid_print(wsrep_xid_uuid(&xid), uuid_str, sizeof(uuid_str));
-  WSREP_INFO("Recovered position: %s:%lld", uuid_str,
-             (long long)wsrep_xid_seqno(&xid));
+  wsrep_uuid_print(&uuid, uuid_str, sizeof(uuid_str));
+  WSREP_INFO("Recovered position: %s:%lld", uuid_str, (long long)seqno);
 }
 
 
@@ -833,19 +778,10 @@ bool wsrep_start_replication()
                               wsrep_sst_donor,
                               bootstrap)))
   {
-    if (-ESOCKTNOSUPPORT == rcode)
-    {
-      DBUG_PRINT("wsrep",("unrecognized cluster address: '%s', rcode: %d",
-                          wsrep_cluster_address, rcode));
-      WSREP_ERROR("unrecognized cluster address: '%s', rcode: %d",
-                  wsrep_cluster_address, rcode);
-    }
-    else
-    {
-      DBUG_PRINT("wsrep",("wsrep->connect() failed: %d", rcode));
-      WSREP_ERROR("wsrep::connect() failed: %d", rcode);
-    }
-
+    DBUG_PRINT("wsrep",("wsrep->connect(%s) failed: %d",
+                        wsrep_cluster_address, rcode));
+    WSREP_ERROR("wsrep::connect(%s) failed: %d",
+                wsrep_cluster_address, rcode);
     return false;
   }
   else
@@ -1320,10 +1256,8 @@ static void wsrep_TOI_end(THD *thd) {
   WSREP_DEBUG("TO END: %lld, %d : %s", (long long)wsrep_thd_trx_seqno(thd),
               thd->wsrep_exec_mode, (thd->query()) ? thd->query() : "void");
   
-  XID xid;
-  wsrep_xid_init(&xid, &thd->wsrep_trx_meta.gtid.uuid,
-                 thd->wsrep_trx_meta.gtid.seqno);
-  wsrep_set_SE_checkpoint(&xid);
+  wsrep_set_SE_checkpoint(thd->wsrep_trx_meta.gtid.uuid,
+                          thd->wsrep_trx_meta.gtid.seqno);
   WSREP_DEBUG("TO END: %lld, update seqno",
               (long long)wsrep_thd_trx_seqno(thd));
   
@@ -1342,12 +1276,30 @@ static int wsrep_RSU_begin(THD *thd, char *db_, char *table_)
   WSREP_DEBUG("RSU BEGIN: %lld, %d : %s", (long long)wsrep_thd_trx_seqno(thd),
                thd->wsrep_exec_mode, thd->query() );
 
+  DEBUG_SYNC(thd,"wsrep_RSU_begin_enter");
+
+  mysql_mutex_lock(&LOCK_wsrep_desync_count);
+
+  DEBUG_SYNC(thd,"wsrep_RSU_begin_after_lock");
+
+  if (wsrep_desync_count++)
+  {
+     mysql_mutex_lock(&LOCK_wsrep_replaying);
+     wsrep_replaying++;
+     mysql_mutex_unlock(&LOCK_wsrep_replaying);
+     mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+     DEBUG_SYNC(thd,"wsrep_RSU_begin_acquired");
+     return 0;
+  }
+
   ret = wsrep->desync(wsrep);
   if (ret != WSREP_OK)
   {
-    WSREP_WARN("RSU desync failed %d for %s", ret, thd->query());
-    my_error(ER_LOCK_DEADLOCK, MYF(0));
-    return(ret);
+     wsrep_desync_count--;
+     mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+     WSREP_WARN("RSU desync failed %d for %s", ret, thd->query());
+     my_error(ER_LOCK_DEADLOCK, MYF(0));
+     return(ret);
   }
   mysql_mutex_lock(&LOCK_wsrep_replaying);
   wsrep_replaying++;
@@ -1360,8 +1312,9 @@ static int wsrep_RSU_begin(THD *thd, char *db_, char *table_)
     mysql_mutex_lock(&LOCK_wsrep_replaying);
     wsrep_replaying--;
     mysql_mutex_unlock(&LOCK_wsrep_replaying);
-
+    wsrep_desync_count--;
     ret = wsrep->resync(wsrep);
+    mysql_mutex_unlock(&LOCK_wsrep_desync_count);
     if (ret != WSREP_OK)
     {
       WSREP_WARN("resync failed %d for %s", ret, thd->query());
@@ -1373,11 +1326,18 @@ static int wsrep_RSU_begin(THD *thd, char *db_, char *table_)
   wsrep_seqno_t seqno = wsrep->pause(wsrep);
   if (seqno == WSREP_SEQNO_UNDEFINED)
   {
+    mysql_mutex_unlock(&LOCK_wsrep_desync_count);
     WSREP_WARN("pause failed %lld for %s", (long long)seqno, thd->query());
     return(1);
   }
-  WSREP_DEBUG("paused at %lld", (long long)seqno);
+  thd->global_read_lock.pause_provider(TRUE);
   thd->variables.wsrep_on = 0;
+  mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+
+  WSREP_DEBUG("paused at %lld", (long long)seqno);
+
+  DEBUG_SYNC(thd,"wsrep_RSU_begin_acquired");
+
   return 0;
 }
 
@@ -1387,23 +1347,31 @@ static void wsrep_RSU_end(THD *thd)
   WSREP_DEBUG("RSU END: %lld, %d : %s", (long long)wsrep_thd_trx_seqno(thd),
                thd->wsrep_exec_mode, thd->query() );
 
+  mysql_mutex_lock(&LOCK_wsrep_desync_count);
 
   mysql_mutex_lock(&LOCK_wsrep_replaying);
   wsrep_replaying--;
   mysql_mutex_unlock(&LOCK_wsrep_replaying);
 
-  ret = wsrep->resume(wsrep);
-  if (ret != WSREP_OK)
+  if (--wsrep_desync_count == 0)
   {
-    WSREP_WARN("resume failed %d for %s", ret, thd->query());
+     ret = wsrep->resume(wsrep);
+     if (ret != WSREP_OK)
+     {
+       WSREP_WARN("resume failed %d for %s", ret, thd->query());
+     }
+     thd->global_read_lock.pause_provider(FALSE);
+     ret = wsrep->resync(wsrep);
+     if (ret != WSREP_OK)
+     {
+       mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+       WSREP_WARN("resync failed %d for %s", ret, thd->query());
+       return;
+     }
+     thd->variables.wsrep_on = 1;
   }
-  ret = wsrep->resync(wsrep);
-  if (ret != WSREP_OK)
-  {
-    WSREP_WARN("resync failed %d for %s", ret, thd->query());
-    return;
-  }
-  thd->variables.wsrep_on = 1;
+
+  mysql_mutex_unlock(&LOCK_wsrep_desync_count);
 }
 
 int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
@@ -1418,36 +1386,59 @@ int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
 
 
   lex= thd->lex;
-  if (!wsrep_replicate_myisam)
+  /*
+    Note: We don't check if enforce_storage_engine succeeds or not
+    We check only the replication requirements here.
+   */
+  if (!thd->variables.wsrep_replicate_myisam && (!enforce_storage_engine ||
+					       (enforce_storage_engine
+						&&
+						strcasecmp
+						(enforce_storage_engine,
+						 "InnoDB")))
+    && lex->sql_command == SQLCOM_CREATE_TABLE)
   {
-    if (table_ && lex->create_info.db_type && \
-        lex->create_info.db_type->db_type == DB_TYPE_MYISAM)
-    {
-        DBUG_ASSERT(db_);
-        WSREP_INFO("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF", db_, table_);
-        return ret;
-    } 
+    if (table_ && lex->create_info.db_type &&
+	lex->create_info.db_type->db_type == DB_TYPE_MYISAM)
+      {
+	if (db_)
+	  {
+	    WSREP_INFO
+	      ("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF",
+	       db_, table_);
+	  }
+	else
+	  {
+	    WSREP_INFO
+	      ("Cannot replicate MyISAM DDL for %s with wsrep_replicate_myisam OFF",
+	       table_);
+	  }
+	return ret;
+      }
     else if (table_list)
-    {
-        for (const TABLE_LIST *table= table_list; table; table= table->next_global)
-        {
-            /*
-             * First condition is required for IF EXISTS queries
-             */
-            if (table->table && table->table->file->ht->db_type == DB_TYPE_MYISAM)
-            {
-                WSREP_INFO("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF", table->db, table->table_name);
-                /* 
-                 * Break even if one table is MyISAM
-                 * Note: This may not work well with CREATE TABLE .. SELECT (CTAS)
-                 * but CTAS is not executed as TOI currently.
-                 */
-                return ret;
-            }
-        }
-    }
+      {
+	for (const TABLE_LIST * table = table_list; table;
+	     table = table->next_global)
+	  {
+	    /*
+	     * First condition is required for IF EXISTS queries
+	     */
+	    if (table->table
+		&& table->table->file->ht->db_type == DB_TYPE_MYISAM)
+	      {
+		WSREP_INFO
+		  ("Cannot replicate MyISAM DDL for %s.%s with wsrep_replicate_myisam OFF",
+		   table->db, table->table_name);
+		/* 
+		 * Break even if one table is MyISAM
+		 * Note: This may not work well with CREATE TABLE .. SELECT (CTAS)
+		 * but CTAS is not executed as TOI currently.
+		 */
+		return ret;
+	      }
+	  }
+      }
   }
-
   mysql_mutex_lock(&thd->LOCK_wsrep_thd);
 
   if (thd->wsrep_conflict_state == MUST_ABORT)
@@ -1469,6 +1460,14 @@ int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
     return -1;
   }
 
+  if (!thd->global_read_lock.provider_resumed())
+  {
+    WSREP_DEBUG("Aborting TOI: Galera provider paused due to lock: %s %lu",
+                thd->query(), thd->thread_id);
+    my_error(ER_CANT_UPDATE_WITH_READLOCK, MYF(0));
+    return -1;
+  }
+
   if (wsrep_debug && thd->mdl_context.has_locks())
   {
     WSREP_DEBUG("thread holds MDL locks at TI begin: %s %lu",
@@ -1490,10 +1489,15 @@ int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
 
   if (thd->variables.wsrep_on && thd->wsrep_exec_mode==LOCAL_STATE)
   {
-    switch (wsrep_OSU_method_options) {
+    switch (thd->variables.wsrep_OSU_method) {
     case WSREP_OSU_TOI: ret =  wsrep_TOI_begin(thd, db_, table_,
                                                table_list); break;
     case WSREP_OSU_RSU: ret =  wsrep_RSU_begin(thd, db_, table_); break;
+    default:
+      WSREP_ERROR("Unsupported OSU method: %lu",
+                  thd->variables.wsrep_OSU_method);
+      ret= -1;
+      break;
     }
     switch (ret) {
     case 0:  thd->wsrep_exec_mode= TOTAL_ORDER; break;
@@ -1513,10 +1517,14 @@ void wsrep_to_isolation_end(THD *thd)
 {
   if (thd->wsrep_exec_mode == TOTAL_ORDER)
   {
-    switch(wsrep_OSU_method_options)
+    switch(thd->variables.wsrep_OSU_method)
     {
     case WSREP_OSU_TOI: wsrep_TOI_end(thd); break;
     case WSREP_OSU_RSU: wsrep_RSU_end(thd); break;
+    default:
+      WSREP_WARN("Unsupported wsrep OSU method at isolation end: %lu",
+                 thd->variables.wsrep_OSU_method);
+      break;
     }
     wsrep_cleanup_transaction(thd);
   }
@@ -1562,12 +1570,13 @@ wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
       mysql_mutex_unlock(&granted_thd->LOCK_wsrep_thd);
       ret = TRUE;
     }
-    else if (granted_thd->lex->sql_command == SQLCOM_FLUSH)
+    else if (granted_thd->lex->sql_command == SQLCOM_FLUSH ||
+             granted_thd->mdl_context.wsrep_has_explicit_locks())
     {
-      WSREP_DEBUG("mdl granted over FLUSH BF");
+      WSREP_DEBUG("BF thread waiting for FLUSH");
       ticket->wsrep_report(wsrep_debug);
       mysql_mutex_unlock(&granted_thd->LOCK_wsrep_thd);
-      ret = TRUE;
+      ret = FALSE;
     }
     else if (request_thd->lex->sql_command == SQLCOM_DROP_TABLE)
     {
