@@ -1,4 +1,4 @@
-/* Copyright 2008 Codership Oy <http://www.codership.com>
+/* Copyright 2008-2015 Codership Oy <http://www.codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <sql_acl.h>
 #include "wsrep_priv.h"
 #include "wsrep_thd.h"
+#include "wsrep_xid.h"
 #include <my_dir.h>
 #include <cstdio>
 #include <cstdlib>
@@ -37,7 +38,6 @@ const  char* wsrep_node_name        = 0;
 const  char* wsrep_node_address     = 0;
 const  char* wsrep_node_incoming_address = 0;
 const  char* wsrep_start_position   = 0;
-ulong  wsrep_OSU_method_options;
 ulong   wsrep_reject_queries_options;
 
 int wsrep_init_vars()
@@ -75,6 +75,13 @@ bool wsrep_causal_reads_update (sys_var *self, THD* thd, enum_var_type var_type)
   } else {
     thd->variables.wsrep_sync_wait &= ~WSREP_SYNC_WAIT_BEFORE_READ;
   }
+
+  // update global settings too.
+  if (global_system_variables.wsrep_causal_reads) {
+      global_system_variables.wsrep_sync_wait |= WSREP_SYNC_WAIT_BEFORE_READ;
+  } else {
+      global_system_variables.wsrep_sync_wait &= ~WSREP_SYNC_WAIT_BEFORE_READ;
+  }
   return false;
 }
 
@@ -86,6 +93,10 @@ bool wsrep_sync_wait_update (sys_var* self, THD* thd, enum_var_type var_type)
   // }
   thd->variables.wsrep_causal_reads = thd->variables.wsrep_sync_wait &
           WSREP_SYNC_WAIT_BEFORE_READ;
+
+  // update global settings too
+  global_system_variables.wsrep_causal_reads = global_system_variables.wsrep_sync_wait &
+          WSREP_SYNC_WAIT_BEFORE_READ;
   return false;
 }
 
@@ -136,29 +147,30 @@ err:
   return 1;
 }
 
-void wsrep_set_local_position (const char* value)
+static
+void wsrep_set_local_position(const char* const value, bool const sst)
 {
-  size_t value_len  = strlen (value);
-  size_t uuid_len   = wsrep_uuid_scan (value, value_len, &local_uuid);
+  size_t const value_len = strlen(value);
+  wsrep_uuid_t uuid;
+  size_t const uuid_len = wsrep_uuid_scan(value, value_len, &uuid);
+  wsrep_seqno_t const seqno = strtoll(value + uuid_len + 1, NULL, 10);
 
-  local_seqno = strtoll (value + uuid_len + 1, NULL, 10);
-
-  XID xid;
-  wsrep_xid_init(&xid, &local_uuid, local_seqno);
-  wsrep_set_SE_checkpoint(&xid);
-  WSREP_INFO ("wsrep_start_position var submitted: '%s'", wsrep_start_position);
+  if (sst) {
+    wsrep_sst_received (wsrep, uuid, seqno, NULL, 0);
+  } else {
+    // initialization
+    local_uuid = uuid;
+    local_seqno = seqno;
+  }
 }
 
 bool wsrep_start_position_update (sys_var *self, THD* thd, enum_var_type type)
 {
+  WSREP_INFO ("wsrep_start_position var submitted: '%s'",
+              wsrep_start_position);
   // since this value passed wsrep_start_position_check, don't check anything
   // here
-  wsrep_set_local_position (wsrep_start_position);
-
-  if (wsrep) {
-    wsrep_sst_received (wsrep, &local_uuid, local_seqno, NULL, 0);
-  }
-
+  wsrep_set_local_position (wsrep_start_position, true);
   return 0;
 }
 
@@ -171,7 +183,7 @@ void wsrep_start_position_init (const char* val)
     return;
   }
 
-  wsrep_set_local_position (val);
+  wsrep_set_local_position (val, false);
 }
 
 static bool refresh_provider_options()
@@ -255,7 +267,13 @@ bool wsrep_provider_update (sys_var *self, THD* thd, enum_var_type type)
   */
   mysql_mutex_unlock(&LOCK_global_system_variables);
   wsrep_stop_replication(thd);
+  /*
+    Unlock and lock LOCK_wsrep_slave_threads to maintain lock order & avoid
+    any potential deadlock.
+  */
+  mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
   mysql_mutex_lock(&LOCK_global_system_variables);
+  mysql_mutex_lock(&LOCK_wsrep_slave_threads);
 
   wsrep_deinit();
 
@@ -381,7 +399,13 @@ bool wsrep_cluster_address_update (sys_var *self, THD* thd, enum_var_type type)
   */
   mysql_mutex_unlock(&LOCK_global_system_variables);
   wsrep_stop_replication(thd);
+  /*
+    Unlock and lock LOCK_wsrep_slave_threads to maintain lock order & avoid
+    any potential deadlock.
+  */
+  mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
   mysql_mutex_lock(&LOCK_global_system_variables);
+  mysql_mutex_lock(&LOCK_wsrep_slave_threads);
 
   if (wsrep_start_replication())
   {
@@ -493,16 +517,9 @@ void wsrep_node_address_init (const char* value)
 
 bool wsrep_slave_threads_check (sys_var *self, THD* thd, set_var* var)
 {
-  if (wsrep_slave_count_change != 0) {
-      WSREP_ERROR("Still closing existing threads - %d", abs(wsrep_slave_count_change));
-     push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                        ER_UNKNOWN_ERROR,
-                        "Still closing existing threads - %d",
-                        abs(wsrep_slave_count_change));
-      return true;
-  }
   mysql_mutex_lock(&LOCK_wsrep_slave_threads);
-  wsrep_slave_count_change = var->value->val_int() - wsrep_slave_threads;
+  wsrep_slave_count_change += (var->save_result.ulonglong_value -
+                               wsrep_slave_threads);
   mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
 
   return false;
@@ -521,43 +538,92 @@ bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
 
 bool wsrep_desync_check (sys_var *self, THD* thd, set_var* var)
 {
-  bool new_wsrep_desync = var->save_result.ulonglong_value; 
-  if (wsrep_desync == new_wsrep_desync) {
-    if (new_wsrep_desync) {
-      WSREP_ERROR("wsrep_desync is already ON.");  
-      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
-                   ER_WRONG_VALUE_FOR_VAR,
-                   "'wsrep_desync' is already ON.");
-      return true;
-    } else {
-      WSREP_ERROR("wsrep_desync is already OFF.");  
-      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
-                   ER_WRONG_VALUE_FOR_VAR,
-                   "'wsrep_desync' is already OFF.");
-      return true;
-    }
+  bool new_wsrep_desync = var->save_result.ulonglong_value;
+  mysql_mutex_lock(&LOCK_wsrep_desync_count);
+  if (new_wsrep_desync)
+  {
+      if (wsrep_desync_count_manual)
+      {
+          WSREP_DEBUG("wsrep_desync is already ON, the counter is increased.");
+      }
   }
+  else
+  {
+      if (wsrep_desync_count_manual == 0)
+      {
+          mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+          WSREP_ERROR ("Trying to make wsrep_desync = OFF on "
+                       "the node that is already synchronized.");
+          my_error (ER_WRONG_VALUE_FOR_VAR, MYF(0), 
+                    var->var->name.str, "OFF");
+          return true;
+      }
+      else if (wsrep_desync_count_manual != 1)
+      {
+          WSREP_DEBUG("wsrep_desync still ON, new desync counter = %d.",
+                      (int) (wsrep_desync_count_manual - 1));
+      }
+  }
+  mysql_mutex_unlock(&LOCK_wsrep_desync_count);
   return 0;
 }
 
 bool wsrep_desync_update (sys_var *self, THD* thd, enum_var_type type)
 {
   wsrep_status_t ret(WSREP_WARNING);
-  if (wsrep_desync) {
-    ret = wsrep->desync (wsrep);
-    if (ret != WSREP_OK) {
-      WSREP_WARN ("SET desync failed %d for %s", ret, thd->query());
-      my_error (ER_CANNOT_USER, MYF(0), "'desync'", thd->query());
-      return true;
-    }
-  } else {
-    ret = wsrep->resync (wsrep);
-    if (ret != WSREP_OK) {
-      WSREP_WARN ("SET resync failed %d for %s", ret, thd->query());
-      my_error (ER_CANNOT_USER, MYF(0), "'resync'", thd->query());
-      return true;
-    }
+  mysql_mutex_lock(&LOCK_wsrep_desync_count);
+  if (wsrep_desync)
+  {
+      wsrep_desync_count_manual++;
+      if (wsrep_desync_count++ == 0)
+      {
+          ret = wsrep->desync (wsrep);
+          if (ret != WSREP_OK)
+          {
+              wsrep_desync_count--;
+              wsrep_desync_count_manual--;
+              mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+              WSREP_WARN ("SET desync failed %d for %s", ret,
+                          thd->query());
+              my_error (ER_CANNOT_USER, MYF(0), "'wsrep->desync()'",
+                        thd->query());
+              return true;
+          }
+      }
   }
+  else
+  {
+      if (wsrep_desync_count_manual)
+      {
+          if (--wsrep_desync_count_manual)
+          {
+              wsrep_desync = 1;
+          }
+          if (--wsrep_desync_count == 0)
+          {
+              ret = wsrep->resync (wsrep);
+              if (ret != WSREP_OK)
+              {
+                  mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+                  WSREP_WARN ("SET resync failed %d for %s", ret,
+                              thd->query());
+                  my_error (ER_CANNOT_USER, MYF(0), "'wsrep->resync()'",
+                            thd->query());
+                  return true;
+              }
+          }
+      }
+      else
+      {
+          mysql_mutex_unlock(&LOCK_wsrep_desync_count);
+          WSREP_ERROR ("Trying to make wsrep_desync = OFF on "
+                       "the node that is already synchronized.");
+          my_error (ER_CANNOT_USER, MYF(0), "'wsrep_desync_update'",
+                    thd->query());
+          return true;
+      }
+  }
+  mysql_mutex_unlock(&LOCK_wsrep_desync_count);
   return false;
 }
 
@@ -605,7 +671,9 @@ static void export_wsrep_status_to_mysql(THD* thd)
 
   for (wsrep_status_len = 0;
        thd->wsrep_status_vars[wsrep_status_len].name != NULL;
-       wsrep_status_len++);
+       wsrep_status_len++) {
+      /* */
+  }
 
 #if DYNAMIC
   if (wsrep_status_len != mysql_status_len) {
