@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2013, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2015, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -294,7 +294,7 @@ trx_create(void)
 	trx->lock.table_locks = ib_vector_create(
 		heap_alloc, sizeof(void**), 32);
 
-#if defined(WITH_WSREP) && not defined(HAVE_ATOMIC_BUILTINS)
+#ifdef WITH_WSREP
 	trx->wsrep_event = NULL;
 #endif /* WITH_WSREP */
  	return(trx);
@@ -472,11 +472,10 @@ trx_free_prepared(
 /*==============*/
 	trx_t*	trx)	/*!< in, own: trx object */
 {
-	ut_ad(mutex_own(&trx_sys->mutex));
-
 	ut_a(trx_state_eq(trx, TRX_STATE_PREPARED));
 	ut_a(trx->magic_n == TRX_MAGIC_N);
 
+	lock_trx_release_locks(trx);
 	trx_undo_free_prepared(trx);
 
 	assert_trx_in_rw_list(trx);
@@ -486,7 +485,9 @@ trx_free_prepared(
 	UT_LIST_REMOVE(trx_list, trx_sys->rw_trx_list, trx);
 	ut_d(trx->in_rw_trx_list = FALSE);
 
+	mutex_enter(&trx_sys->mutex);
 	trx_release_descriptor(trx);
+	mutex_exit(&trx_sys->mutex);
 
 	/* Undo trx_resurrect_table_locks(). */
 	UT_LIST_INIT(trx->lock.trx_locks);
@@ -1241,11 +1242,11 @@ trx_write_serialisation_history(
 		trx_sys_update_mysql_binlog_offset(
 			trx->mysql_log_file_name,
 			trx->mysql_log_offset,
-			TRX_SYS_MYSQL_LOG_INFO,
 #ifdef WITH_WSREP
-                        sys_header,
+			TRX_SYS_MYSQL_LOG_INFO, sys_header, mtr);
+#else
+			TRX_SYS_MYSQL_LOG_INFO, mtr);
 #endif /* WITH_WSREP */
-                        mtr);
 
 		trx->mysql_log_file_name = NULL;
 	}
@@ -1516,6 +1517,12 @@ trx_commit_in_memory(
 		}
 
 		trx->commit_lsn = lsn;
+
+		/* Tell server some activity has happened, since the trx
+		does changes something. Background utility threads like
+		master thread, purge thread or page_cleaner thread might
+		have some work to do. */
+		srv_active_wake_master_thread();
 	}
 
 	/* undo_no is non-zero if we're doing the final commit. */
@@ -2574,6 +2581,10 @@ trx_start_for_ddl_low(
 		trx->will_lock = 1;
 
 		trx->ddl = true;
+#ifdef WITH_WSREP
+		ut_d(trx->start_file = __FILE__);
+		ut_d(trx->start_line = __LINE__);
+#endif /* WITH_WSREP */
 
 #ifdef WITH_WSREP
 		ut_d(trx->start_file = __FILE__);
