@@ -25,6 +25,15 @@
 # Description: Percona-Server is a SQL database engine with focus on high performance.
 ### END INIT INFO
  
+# Prevent OpenSUSE's init scripts from calling systemd, so that
+# both 'bootstrap' and 'start' are handled entirely within this script
+
+SYSTEMD_NO_WRAP=1
+
+# Prevent Debian's init scripts from calling systemctl
+
+_SYSTEMCTL_SKIP_REDIRECT=true
+ 
 # If you install MySQL on some other places than @prefix@, then you
 # have to do one of the following things for this script to work:
 #
@@ -43,6 +52,13 @@
 # If you change base dir, you must also change datadir. These may get
 # overwritten by settings in the MySQL configuration files.
 
+# source the defauts file /etc/sysconfig/mysql if it exists for any environment variables.
+# set -a is done so that  'FOO=BAR' just be used than 'export FOO=BAR' in the /etc/sysconfig/mysql.
+# However, export FOO=BAR in that file also should work.
+set -a
+[ -r /etc/sysconfig/mysql ] && . /etc/sysconfig/mysql
+set +a
+
 basedir=
 datadir=
 
@@ -443,6 +459,12 @@ case "$mode" in
       echo $echo_n "Bootstrapping PXC (Percona XtraDB Cluster)"
       $0 start $other_args --wsrep-new-cluster
       ;;
+    'bootstrap')
+      # Bootstrap the cluster, start the first node
+      # that initiate the cluster
+      echo $echo_n "Bootstrapping the cluster"
+      $0 start $other_args --wsrep-new-cluster
+      ;;
     *)
       # usage
       basename=`basename "$0"`
