@@ -1,4 +1,4 @@
-# Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,13 +25,15 @@
 %global mysql_vendor            Oracle and/or its affiliates
 
 %global mysql_version   @VERSION@
+%global wsrep_version   @WSREP_VERSION@
+%global wsrep_revision  @WSREP_REVISION@
 
 %global mysqld_user     mysql
 %global mysqld_group    mysql
 %global mysqldatadir    /var/lib/mysql
 
-%global release         1  
-
+%global release         %{wsrep_version}
+%global short_product_tag 5.6
 
 #
 # Macros we use which are not available in all supported versions of RPM
@@ -68,14 +70,6 @@
 #   $ rpmbuild --define="option <x>" ...
 #
 
-# ----------------------------------------------------------------------------
-# wsrep builds
-# ----------------------------------------------------------------------------
-%if %{defined with_wsrep}
-%define mysql_version @VERSION@_wsrep_@WSREP_API_VERSION@.@WSREP_PATCH_VERSION@
-%define wsrep_version @WSREP_VERSION@
-%endif
-
 # ----------------------------------------------------------------------------
 # Commercial builds
 # ----------------------------------------------------------------------------
@@ -87,9 +81,9 @@
 # Source name
 # ----------------------------------------------------------------------------
 %if %{undefined src_base}
-%define src_base mysql
+%define src_base mysql-wsrep
 %endif
-%define src_dir %{src_base}-%{mysql_version}
+%define src_dir %{src_base}-%{mysql_version}-%{wsrep_version}
 
 # ----------------------------------------------------------------------------
 # Feature set (storage engines, options).  Default to community (everything)
@@ -123,105 +117,51 @@
 %define server_suffix   %{nil}
 %endif
 
-# ----------------------------------------------------------------------------
-# Packager
-# ----------------------------------------------------------------------------
-%if %{undefined mysql_packager}
-%define mysql_packager MySQL Build Team <build@mysql.com>
-%endif
-
 # ----------------------------------------------------------------------------
 # Distribution support
 # ----------------------------------------------------------------------------
-%if %{undefined distro_specific}
-%define distro_specific 0
+
+# Disable post build checks for time being.
+BuildConflicts: post-build-checks
+
+BuildRequires: gcc-c++ ncurses-devel perl zlib-devel cmake libaio-devel bison flex
+
+%if 0%{?rhel} == 6 || 0%{?rhel} == 7 || 0%{?fedora} == 20 || 0%{?fedora} == 21
+BuildRequires: time
 %endif
-%if %{distro_specific}
-  %if %(test -f /etc/enterprise-release && echo 1 || echo 0)
-    %define oelver %(rpm -qf --qf '%%{version}\\n' /etc/enterprise-release | sed -e 's/^\\([0-9]*\\).*/\\1/g')
-    %if "%oelver" == "4"
-      %define distro_description        Oracle Enterprise Linux 4
-      %define distro_releasetag         oel4
-      %define distro_buildreq           gcc-c++ gperf ncurses-devel perl time zlib-devel cmake libaio-devel
-      %define distro_requires           chkconfig coreutils grep procps shadow-utils net-tools
-    %else
-      %if "%oelver" == "5"
-        %define distro_description      Oracle Enterprise Linux 5
-        %define distro_releasetag       oel5
-        %define distro_buildreq         gcc-c++ gperf ncurses-devel perl time zlib-devel cmake libaio-devel
-        %define distro_requires         chkconfig coreutils grep procps shadow-utils net-tools
-      %else
-        %{error:Oracle Enterprise Linux %{oelver} is unsupported}
+
+%if 0%{?suse_version}
+%if 0%{?suse_version} == 1110
+BuildRequires: gdbm-devel gperf openldap2-client procps pwdutils
       %endif
+%if 0%{?suse_version} == 1310 || 0%{?suse_version} == 1315 || 0%{?suse_version} == 1320
+BuildRequires: gperf procps time
     %endif
-  %else
-    %if %(test -f /etc/oracle-release && echo 1 || echo 0)
-      %define elver %(rpm -qf --qf '%%{version}\\n' /etc/oracle-release | sed -e 's/^\\([0-9]*\\).*/\\1/g')
-      %if "%elver" == "6" || "%elver" == "7"
-        %define distro_description      Oracle Linux %elver
-        %define distro_releasetag       el%elver
-        %define distro_buildreq         gcc-c++ ncurses-devel perl time zlib-devel cmake libaio-devel
-        %define distro_requires         chkconfig coreutils grep procps shadow-utils net-tools
-      %else
-        %{error:Oracle Linux %{elver} is unsupported}
       %endif
-    %else
-      %if %(test -f /etc/redhat-release && echo 1 || echo 0)
-        %define rhelver %(rpm -qf --qf '%%{version}\\n' /etc/redhat-release | sed -e 's/^\\([0-9]*\\).*/\\1/g')
-        %if "%rhelver" == "4"
-          %define distro_description      Red Hat Enterprise Linux 4
-          %define distro_releasetag       rhel4
-          %define distro_buildreq         gcc-c++ gperf ncurses-devel perl time zlib-devel cmake libaio-devel
-          %define distro_requires         chkconfig coreutils grep procps shadow-utils net-tools
-        %else
-          %if "%rhelver" == "5"
-            %define distro_description    Red Hat Enterprise Linux 5
-            %define distro_releasetag     rhel5
-            %define distro_buildreq       gcc-c++ gperf ncurses-devel perl time zlib-devel cmake libaio-devel
-            %define distro_requires       chkconfig coreutils grep procps shadow-utils net-tools
-          %else
-            %if "%rhelver" == "6"
-              %define distro_description    Red Hat Enterprise Linux 6
-              %define distro_releasetag     rhel6
-              %define distro_buildreq       gcc-c++ ncurses-devel perl time zlib-devel cmake libaio-devel
-              %define distro_requires       chkconfig coreutils grep procps shadow-utils net-tools
-            %else
-              %{error:Red Hat Enterprise Linux %{rhelver} is unsupported}
+
+# Define dist tag if not given by platform
+%if %{undefined dist}
+  # For suse versions see:
+  # https://en.opensuse.org/openSUSE:Build_Service_cross_distribution_howto
+  %if 0%{?suse_version} == 1110
+    %define dist .sle11
             %endif
+  %if 0%{?suse_version} == 1310
+    %define dist .suse13.1
           %endif
+  %if 0%{?suse_version} == 1315
+    %define dist .sle12
         %endif
-      %else
-        %if %(test -f /etc/SuSE-release && echo 1 || echo 0)
-          %define susever %(rpm -qf --qf '%%{version}\\n' /etc/SuSE-release | cut -d. -f1)
-          %if "%susever" == "10"
-            %define distro_description    SUSE Linux Enterprise Server 10
-            %define distro_releasetag     sles10
-            %define distro_buildreq       gcc-c++ gdbm-devel gperf ncurses-devel openldap2-client zlib-devel cmake libaio-devel
-            %define distro_requires       aaa_base coreutils grep procps pwdutils
-          %else
-            %if "%susever" == "11"
-              %define distro_description  SUSE Linux Enterprise Server 11
-              %define distro_releasetag   sles11
-              %define distro_buildreq     gcc-c++ gdbm-devel gperf ncurses-devel openldap2-client procps pwdutils zlib-devel cmake libaio-devel
-              %define distro_requires     aaa_base coreutils grep procps pwdutils
-            %else
-              %{error:SuSE %{susever} is unsupported}
+  %if 0%{?suse_version} == 1320
+    %define dist .suse13.2
             %endif
+  # Still missing?
+  %if %{undefined dist}
+    %define dist .DIST
           %endif
-        %else
-          %{error:Unsupported distribution}
-        %endif
-      %endif
-    %endif
-  %endif
-%else
-  %define glibc_version %(/lib/libc.so.6 | grep stable | cut -d, -f1 | cut -c38-)
-  %define distro_description            Generic Linux (glibc %{glibc_version})
-  %define distro_releasetag             linux_glibc%{glibc_version}
-  %define distro_buildreq               gcc-c++ gperf ncurses-devel perl  time zlib-devel
-  %define distro_requires               coreutils grep procps /sbin/chkconfig /usr/sbin/useradd /usr/sbin/groupadd
 %endif
 
+
 # Avoid debuginfo RPMs, leaves binaries unstripped
 %define debug_package   %{nil}
 
@@ -246,7 +186,7 @@
 %define license_files_server    %{src_dir}/LICENSE.mysql
 %define license_type            Commercial
 %else
-%define license_files_server    %{src_dir}/COPYING %{src_dir}/README
+%define license_files_server    COPYING README
 %define license_type            GPL
 %endif
 
@@ -258,17 +198,20 @@ Name:           MySQL%{product_suffix}
 Summary:        MySQL: a very fast and reliable SQL database server
 Group:          Applications/Databases
 Version:        @MYSQL_RPM_VERSION@
-Release:        %{release}%{?distro_releasetag:.%{distro_releasetag}}
-Distribution:   %{distro_description}
+Release:        %{release}%{dist}
+# Distribution:   %{distro_description}
 License:        Copyright (c) 2000, @MYSQL_COPYRIGHT_YEAR@, %{mysql_vendor}. All rights reserved. Under %{license_type} license as shown in the Description field.
-Source:         http://www.mysql.com/Downloads/MySQL-@MYSQL_BASE_VERSION@/%{src_dir}.tar.gz
+Source:         %{src_dir}.tar.gz
+Source99:       mysql-rpmlintrc
+Patch0:          cmake-no-wix.patch
 URL:            http://www.mysql.com/
-Packager:       MySQL Release Engineering <mysql-build@oss.oracle.com> 
+Packager:       Codership Oy <info@galeracluster.com>
 Vendor:         %{mysql_vendor}
-BuildRequires:  %{distro_buildreq}
+# BuildRequires:  %{distro_buildreq}
+#wsrep_patch_tag
 
 # Regression tests may take a long time, override the default to skip them 
-%{!?runselftest:%global runselftest 1}
+%{!?runselftest:%global runselftest 0}
 
 # Think about what you use here since the first step is to
 # run a rm -rf
@@ -297,18 +240,41 @@ documentation and the manual for more information.
 # Sub package definition
 ##############################################################################
 
-%package -n MySQL-server%{product_suffix}
-%if %{defined with_wsrep}
-Version:        %{mysql_version}
-#Release:        %{wsrep_version}.%{release}
-%endif
+%package -n mysql-wsrep%{product_suffix}
+Summary:        MySQL: meta package for a server+client setup
+Group:          Applications/Databases
+Requires:       mysql-wsrep-server%{product_suffix}
+Requires:       mysql-wsrep-client%{product_suffix}
+
+%description -n mysql-wsrep%{product_suffix}
+This meta package ensures the installation of a MySQL server and the necessary
+client programs for operation and administration. It does not itself
+contain those files but rather causes the installation of the subpackages
+"mysql-wsrep-server%{product_suffix}" and "mysql-wsrep-client%{product_suffix}".
+As indicated in the name, the server is built with the "wsrep" plugin so that
+it can be a node in a MySQL Galera Cluster.
+
+# ----------------------------------------------------------------------------
+
+%package -n mysql-wsrep-server%{product_suffix}
 Summary:        MySQL: a very fast and reliable SQL database server
 Group:          Applications/Databases
-%if %{defined with_wsrep}
-Requires:       %{distro_requires} rsync lsof
-%else
-Requires:       %{distro_requires}
+# Distro requirements
+# RedHat
+%if 0%{?fedora} || 0%{?rhel}
+Requires:       chkconfig coreutils grep procps shadow-utils net-tools rsync lsof
+%if 0%{?rhel} == 7 || 0%{?fedora} >= 20
+Requires: perl-Data-Dumper
 %endif
+%endif
+# SUSE
+%if 0%{?suse_version}
+Requires: aaa_base coreutils grep procps rsync lsof
+%if 0%{suse_version} == 1110
+Requires: pwdutils
+%endif
+%endif
+
 %if 0%{?commercial}
 Obsoletes:      MySQL-server
 %else
@@ -321,7 +287,7 @@ Obsoletes:      MySQL-server-advanced-gpl MySQL-server-enterprise-gpl
 Provides:       mysql-server = %{version}-%{release}
 Provides:       mysql-server%{?_isa} = %{version}-%{release}
 
-%description -n MySQL-server%{product_suffix}
+%description -n mysql-wsrep-server%{product_suffix}
 The MySQL(TM) software delivers a very fast, multi-threaded, multi-user,
 and robust SQL (Structured Query Language) database server. MySQL Server
 is intended for mission-critical, heavy-load production systems as well
@@ -342,14 +308,13 @@ and the manual for more information.
 This package includes the MySQL server binary as well as related utilities
 to run and administer a MySQL server.
 
-%if %{defined with_wsrep}
 Built with wsrep patch %{wsrep_version}.
-%endif
+
 If you want to access and work with the database, you have to install
-package "MySQL-client%{product_suffix}" as well!
+package "mysql-wsrep-client%{product_suffix}" as well!
 
 # ----------------------------------------------------------------------------
-%package -n MySQL-client%{product_suffix}
+%package -n mysql-wsrep-client%{product_suffix}
 Summary:        MySQL - Client
 Group:          Applications/Databases
 %if 0%{?commercial}
@@ -364,13 +329,13 @@ Obsoletes:      MySQL-client-advanced-gpl MySQL-client-enterprise-gpl
 Provides:       mysql = %{version}-%{release} 
 Provides:       mysql%{?_isa} = %{version}-%{release}
 
-%description -n MySQL-client%{product_suffix}
+%description -n mysql-wsrep-client%{product_suffix}
 This package contains the standard MySQL clients and administration tools.
 
 For a description of MySQL see the base MySQL RPM or http://www.mysql.com/
 
 # ----------------------------------------------------------------------------
-%package -n MySQL-test%{product_suffix}
+%package -n mysql-wsrep-test%{product_suffix}
 Summary:        MySQL - Test suite
 Group:          Applications/Databases
 %if 0%{?commercial}
@@ -388,13 +353,13 @@ Provides:       mysql-test = %{version}-%{release}
 Provides:       mysql-test%{?_isa} = %{version}-%{release}
 AutoReqProv:    no
 
-%description -n MySQL-test%{product_suffix}
+%description -n mysql-wsrep-test%{product_suffix}
 This package contains the MySQL regression test suite.
 
 For a description of MySQL see the base MySQL RPM or http://www.mysql.com/
 
 # ----------------------------------------------------------------------------
-%package -n MySQL-devel%{product_suffix}
+%package -n mysql-wsrep-devel%{product_suffix}
 Summary:        MySQL - Development header files and libraries
 Group:          Applications/Databases
 %if 0%{?commercial}
@@ -409,14 +374,14 @@ Obsoletes:      MySQL-devel-advanced-gpl MySQL-devel-enterprise-gpl
 Provides:       mysql-devel = %{version}-%{release}
 Provides:       mysql-devel%{?_isa} = %{version}-%{release}
 
-%description -n MySQL-devel%{product_suffix}
+%description -n mysql-wsrep-devel%{product_suffix}
 This package contains the development header files and libraries necessary
 to develop MySQL client applications.
 
 For a description of MySQL see the base MySQL RPM or http://www.mysql.com/
 
 # ----------------------------------------------------------------------------
-%package -n MySQL-shared%{product_suffix}
+%package -n mysql-wsrep-shared%{product_suffix}
 Summary:        MySQL - Shared libraries
 Group:          Applications/Databases
 %if 0%{?commercial}
@@ -430,12 +395,11 @@ Obsoletes:      MySQL-shared-pro-gpl-cert
 Obsoletes:      MySQL-shared-classic MySQL-shared-community MySQL-shared-enterprise
 Obsoletes:      MySQL-shared-advanced-gpl MySQL-shared-enterprise-gpl
 
-%description -n MySQL-shared%{product_suffix}
+%description -n mysql-wsrep-shared%{product_suffix}
 This package contains the shared libraries (*.so*) which certain languages
 and applications need to dynamically load and use MySQL.
 
 # ----------------------------------------------------------------------------
-%if %{undefined with_wsrep}
 %package -n MySQL-embedded%{product_suffix}
 Summary:        MySQL - Embedded library
 Group:          Applications/Databases
@@ -452,7 +416,7 @@ Obsoletes:      MySQL-embedded-pro
 Obsoletes:      MySQL-embedded-classic MySQL-embedded-community MySQL-embedded-enterprise
 Obsoletes:      MySQL-embedded-advanced-gpl MySQL-embedded-enterprise-gpl
 Provides:       mysql-embedded = %{version}-%{release}
-Provides:       mysql-emdedded%{?_isa} = %{version}-%{release}
+Provides:       mysql-embedded%{?_isa} = %{version}-%{release}
 
 %description -n MySQL-embedded%{product_suffix}
 This package contains the MySQL server as an embedded library.
@@ -465,11 +429,14 @@ The API is identical for the embedded MySQL version and the
 client/server version.
 
 For a description of MySQL see the base MySQL RPM or http://www.mysql.com/
-%endif
 
 ##############################################################################
 %prep
-%setup -T -a 0 -c -n %{src_dir}
+%setup -n %{src_dir}
+# That patch is needed with old cmake only, on SLES 11, but it won't do any harm
+# outside Windows, so it may be used in all RPM builds.
+%patch0 -p1
+#wsrep_apply_patch_tag
 ##############################################################################
 %build
 
@@ -509,7 +476,7 @@ export CFLAGS=${MYSQL_BUILD_CFLAGS:-${CFLAGS:-$RPM_OPT_FLAGS}}
 export CXXFLAGS=${MYSQL_BUILD_CXXFLAGS:-${CXXFLAGS:-$RPM_OPT_FLAGS -felide-constructors}}
 export LDFLAGS=${MYSQL_BUILD_LDFLAGS:-${LDFLAGS:-}}
 export CMAKE=${MYSQL_BUILD_CMAKE:-${CMAKE:-cmake}}
-export MAKE_JFLAG=${MYSQL_BUILD_MAKE_JFLAG:-}
+export MAKE_JFLAG=${MYSQL_BUILD_MAKE_JFLAG:--j$(ncpu=$(cat /proc/cpuinfo | grep processor | wc -l) && echo $(($ncpu > 4 ? 4 : $ncpu)))}
 
 # By default, a build will include the bundeled "yaSSL" library for SSL.
 # However, there may be a need to override.
@@ -528,6 +495,7 @@ mkdir debug
   CFLAGS=`echo " ${CFLAGS} " | \
             sed -e 's/ -O[0-9]* / /' \
                 -e 's/-Wp,-D_FORTIFY_SOURCE=2/ /' \
+                -e 's/-D_FORTIFY_SOURCE=2/ /' \
                 -e 's/ -unroll2 / /' \
                 -e 's/ -ip / /' \
                 -e 's/^ //' \
@@ -535,22 +503,23 @@ mkdir debug
   CXXFLAGS=`echo " ${CXXFLAGS} " | \
               sed -e 's/ -O[0-9]* / /' \
                   -e 's/-Wp,-D_FORTIFY_SOURCE=2/ /' \
+                  -e 's/-D_FORTIFY_SOURCE=2/ /' \
                   -e 's/ -unroll2 / /' \
                   -e 's/ -ip / /' \
                   -e 's/^ //' \
                   -e 's/ $//'`
   # XXX: MYSQL_UNIX_ADDR should be in cmake/* but mysql_version is included before
   # XXX: install_layout so we can't just set it based on INSTALL_LAYOUT=RPM
-  ${CMAKE} ../%{src_dir} -DBUILD_CONFIG=mysql_release -DINSTALL_LAYOUT=RPM \
+  ${CMAKE} ../ -DBUILD_CONFIG=mysql_release -DINSTALL_LAYOUT=RPM \
            -DCMAKE_BUILD_TYPE=Debug \
            -DMYSQL_UNIX_ADDR="%{mysqldatadir}/mysql.sock" \
            -DFEATURE_SET="%{feature_set}" \
            %{ssl_option} \
            -DCOMPILATION_COMMENT="%{compilation_comment_debug}" \
-%if %{defined with_wsrep}
-           -DWITH_WSREP=1 -DWITH_LIBEVENT=system \
-%endif
-           -DMYSQL_SERVER_SUFFIX="%{server_suffix}"
+           -DMYSQL_SERVER_SUFFIX="%{server_suffix}" \
+           -DWITH_WSREP=1 \
+           -DWSREP_VERSION="%{wsrep_version}" \
+           -DWSREP_REVISION="%{wsrep_revision}"
   echo BEGIN_DEBUG_CONFIG ; egrep '^#define' include/config.h ; echo END_DEBUG_CONFIG
   make ${MAKE_JFLAG} VERBOSE=1
 )
@@ -560,16 +529,16 @@ mkdir release
   cd release
   # XXX: MYSQL_UNIX_ADDR should be in cmake/* but mysql_version is included before
   # XXX: install_layout so we can't just set it based on INSTALL_LAYOUT=RPM
-  ${CMAKE} ../%{src_dir} -DBUILD_CONFIG=mysql_release -DINSTALL_LAYOUT=RPM \
+  ${CMAKE} ../ -DBUILD_CONFIG=mysql_release -DINSTALL_LAYOUT=RPM \
            -DCMAKE_BUILD_TYPE=RelWithDebInfo \
            -DMYSQL_UNIX_ADDR="%{mysqldatadir}/mysql.sock" \
            -DFEATURE_SET="%{feature_set}" \
            %{ssl_option} \
            -DCOMPILATION_COMMENT="%{compilation_comment_release}" \
-%if %{defined with_wsrep}
-           -DWITH_WSREP=1 -DWITH_LIBEVENT=system \
-%endif
-           -DMYSQL_SERVER_SUFFIX="%{server_suffix}"
+           -DMYSQL_SERVER_SUFFIX="%{server_suffix}" \
+           -DWITH_WSREP=1 \
+           -DWSREP_VERSION="%{wsrep_version}" \
+           -DWSREP_REVISION="%{wsrep_revision}"
   echo BEGIN_NORMAL_CONFIG ; egrep '^#define' include/config.h ; echo END_NORMAL_CONFIG
   make ${MAKE_JFLAG} VERBOSE=1
 )
@@ -618,21 +587,18 @@ install -m 755 $MBD/release/support-files/mysql.server $RBR%{_sysconfdir}/init.d
 # will appreciate that, as all services usually offer this.
 ln -sf %{_sysconfdir}/init.d/mysql $RBR%{_sbindir}/rcmysql
 
-%if %{defined with_wsrep}
 # Create a wsrep_sst_rsync_wan symlink.
 install -d $RBR%{_bindir}
 ln -sf wsrep_sst_rsync $RBR%{_bindir}/wsrep_sst_rsync_wan
-%endif
 
 # Touch the place where the my.cnf config file might be located
 # Just to make sure it's in the file list and marked as a config file
 touch $RBR%{_sysconfdir}/my.cnf
-%if %{defined with_wsrep}
 touch $RBR%{_sysconfdir}/wsrep.cnf
-%endif
+
 
 # Install SELinux files in datadir
-install -m 600 $MBD/%{src_dir}/support-files/RHEL4-SElinux/mysql.{fc,te} \
+install -m 600 $MBD/support-files/RHEL4-SElinux/mysql.{fc,te} \
   $RBR%{_datadir}/mysql/SELinux/RHEL4
 
 %if %{WITH_TCMALLOC}
@@ -651,7 +617,7 @@ install -m 644 "%{malloc_lib_source}" \
 #  Post processing actions, i.e. when installed
 ##############################################################################
 
-%pre -n MySQL-server%{product_suffix}
+%pre -n mysql-wsrep-server%{product_suffix}
 # This is the code running at the beginning of a RPM upgrade action,
 # before replacing the old files with the new ones.
 
@@ -736,7 +702,7 @@ A manual upgrade is required.
   rpm -qa | grep -i '^mysql-'
 
   You may choose to use 'rpm --nodeps -ev <package-name>' to remove
-  the package which contains the mysqlclient shared library.  The
+  the package which contains the perconaserverclient shared library.  The
   library will be reinstalled by the MySQL-shared-compat package.
 - Install the new MySQL packages supplied by $myvendor
 - Ensure that the MySQL server is started
@@ -829,7 +795,7 @@ if [ -x %{_sysconfdir}/init.d/mysql ] ; then
         sleep 5
 fi
 
-%post -n MySQL-server%{product_suffix}
+%post -n mysql-wsrep-server%{product_suffix}
 # This is the code running at the end of a RPM install or upgrade action,
 # after the (new) files have been written.
 
@@ -989,7 +955,7 @@ mv -f  $STATUS_FILE ${STATUS_FILE}-LAST  # for "triggerpostun"
 #scheduled service packs and more.  Visit www.mysql.com/enterprise for more
 #information."
 
-%preun -n MySQL-server%{product_suffix}
+%preun -n mysql-wsrep-server%{product_suffix}
 
 # Which '$1' does this refer to?  Fedora docs have info:
 # " ... a count of the number of versions of the package that are installed.
@@ -1018,7 +984,7 @@ fi
 # We do not remove the mysql user since it may still own a lot of
 # database files.
 
-%triggerpostun -n MySQL-server%{product_suffix} --MySQL-server-community
+%triggerpostun -n mysql-wsrep-server%{product_suffix} --MySQL-server-community
 
 # Setup: We renamed this package, so any existing "server-community"
 #   package will be removed when this "server" is installed.
@@ -1089,20 +1055,22 @@ echo "====="                                     >> $STATUS_HISTORY
 #  Files section
 ##############################################################################
 
-%files -n MySQL-server%{product_suffix} -f release/support-files/plugins.files
+%files -n mysql-wsrep%{product_suffix}
+# Intentionally empty - this is a pure meta package.
+
+# ----------------------------------------------------------------------------
+%files -n mysql-wsrep-server%{product_suffix} -f release/support-files/plugins.files
 %defattr(-,root,root,0755)
 %if %{defined license_files_server}
 %doc %{license_files_server}
 %endif
-%doc %{src_dir}/Docs/ChangeLog
-%doc %{src_dir}/Docs/INFO_SRC*
+%doc Docs/ChangeLog
+%doc release/Docs/INFO_SRC*
 %doc release/Docs/INFO_BIN*
 %doc release/support-files/my-default.cnf
-%if %{defined with_wsrep}
-%doc %{src_dir}/Docs/README-wsrep
+%doc Docs/README-wsrep
 %doc release/support-files/wsrep.cnf
 %doc release/support-files/wsrep_notify
-%endif
 
 %if 0%{?commercial}
 %doc %attr(644, root, root) %{_infodir}/mysql.info*
@@ -1138,6 +1106,7 @@ echo "====="                                     >> $STATUS_HISTORY
 %doc %attr(644, root, man) %{_mandir}/man1/resolveip.1*
 
 %ghost %config(noreplace,missingok) %{_sysconfdir}/my.cnf
+%ghost %config(noreplace,missingok) %{_sysconfdir}/wsrep.cnf
 %dir %{_sysconfdir}/my.cnf.d
 
 %attr(755, root, root) %{_bindir}/innochecksum
@@ -1165,18 +1134,18 @@ echo "====="                                     >> $STATUS_HISTORY
 %attr(755, root, root) %{_bindir}/replace
 %attr(755, root, root) %{_bindir}/resolve_stack_dump
 %attr(755, root, root) %{_bindir}/resolveip
-%if %{defined with_wsrep}
 %attr(755, root, root) %{_bindir}/wsrep_sst_common
 %attr(755, root, root) %{_bindir}/wsrep_sst_mysqldump
 %attr(755, root, root) %{_bindir}/wsrep_sst_rsync
 %attr(755, root, root) %{_bindir}/wsrep_sst_rsync_wan
 %attr(755, root, root) %{_bindir}/wsrep_sst_xtrabackup
 %attr(755, root, root) %{_bindir}/wsrep_sst_xtrabackup-v2
-%endif
 
 %attr(755, root, root) %{_sbindir}/mysqld
 %attr(755, root, root) %{_sbindir}/mysqld-debug
 %attr(755, root, root) %{_sbindir}/rcmysql
+%dir %{_libdir}/mysql/plugin
+%dir %{_libdir}/mysql/plugin/debug
 %attr(755, root, root) %{_libdir}/mysql/plugin/daemon_example.ini
 
 %if %{WITH_TCMALLOC}
@@ -1189,7 +1158,8 @@ echo "====="                                     >> $STATUS_HISTORY
 %dir %attr(755, mysql, mysql) /var/lib/mysql
 
 # ----------------------------------------------------------------------------
-%files -n MySQL-client%{product_suffix}
+%files -n mysql-wsrep-client%{product_suffix}
+
 %defattr(-, root, root, 0755)
 %if %{defined license_files_server}
 %doc %{license_files_server}
@@ -1225,7 +1195,7 @@ echo "====="                                     >> $STATUS_HISTORY
 %doc %attr(644, root, man) %{_mandir}/man1/mysql_config_editor.1*
 
 # ----------------------------------------------------------------------------
-%files -n MySQL-devel%{product_suffix} -f optional-files-devel
+%files -n mysql-wsrep-devel%{product_suffix} -f optional-files-devel
 %defattr(-, root, root, 0755)
 %if %{defined license_files_server}
 %doc %{license_files_server}
@@ -1237,12 +1207,12 @@ echo "====="                                     >> $STATUS_HISTORY
 %dir %attr(755, root, root) %{_libdir}/mysql
 %{_includedir}/mysql/*
 %{_datadir}/aclocal/mysql.m4
-%{_libdir}/mysql/libmysqlclient.a
-%{_libdir}/mysql/libmysqlclient_r.a
+%{_libdir}/mysql/libperconaserverclient.a
+%{_libdir}/mysql/libperconaserverclient_r.a
 %{_libdir}/mysql/libmysqlservices.a
 
 # ----------------------------------------------------------------------------
-%files -n MySQL-shared%{product_suffix}
+%files -n mysql-wsrep-shared%{product_suffix}
 %defattr(-, root, root, 0755)
 %if %{defined license_files_server}
 %doc %{license_files_server}
@@ -1250,48 +1220,47 @@ echo "====="                                     >> $STATUS_HISTORY
 # Shared libraries (omit for architectures that don't support them)
 %{_libdir}/libmysql*.so*
 
-%post -n MySQL-shared%{product_suffix}
+%post -n mysql-wsrep-shared%{product_suffix}
 /sbin/ldconfig
 
-%postun -n MySQL-shared%{product_suffix}
+%postun -n mysql-wsrep-shared%{product_suffix}
 /sbin/ldconfig
 
 # ----------------------------------------------------------------------------
-%files -n MySQL-test%{product_suffix}
+%files -n mysql-wsrep-test%{product_suffix}
 %defattr(-, root, root, 0755)
 %if %{defined license_files_server}
 %doc %{license_files_server}
 %endif
 %attr(-, root, root) %{_datadir}/mysql-test
 %attr(755, root, root) %{_bindir}/mysql_client_test
-%if %{undefined with_wsrep}
-%attr(755, root, root) %{_bindir}/mysql_client_test_embedded
-%attr(755, root, root) %{_bindir}/mysqltest_embedded
-%endif
 %doc %attr(644, root, man) %{_mandir}/man1/mysql_client_test.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql-stress-test.pl.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql-test-run.pl.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql_client_test_embedded.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqltest_embedded.1*
 
-# ----------------------------------------------------------------------------
-%if %{undefined with_wsrep}
-%files -n MySQL-embedded%{product_suffix}
-%defattr(-, root, root, 0755)
-%if %{defined license_files_server}
-%doc %{license_files_server}
-%endif
-%attr(755, root, root) %{_bindir}/mysql_embedded
-%attr(644, root, root) %{_libdir}/mysql/libmysqld.a
-%attr(644, root, root) %{_libdir}/mysql/libmysqld-debug.a
-%endif
-
 ##############################################################################
 # The spec file changelog only includes changes made to the spec file
 # itself - note that they must be ordered by date (important when
 # merging BK trees)
 ##############################################################################
 %changelog
+* Thu Jan 29 2015 Joerg Bruehe <joerg.bruehe@fromdual.com>
+- Add a meta-package "mysql-wsrep" that requires both "server" and "client".
+- Fix the fall-back definition of "dist", it must start with a period.
+
+* Mon Jan 26 2015 Joerg Bruehe <joerg.bruehe@fromdual.com>
+- Allow "rpmlint", but suppress "post-build-checks" (fail on SuSE 12 + 13).
+- Improve handling of undefined "%%{dist}".
+- Fix wrong changelog dates, to get rid of warnings about "bogus date".
+- Escape percent signs in changelog, to get rid of "rpmlint" warnings.
+
+* Tue Jan 20 2015 Teemu Ollakka <teemu.ollakka@galeracluster.com>
+
+- Reworked to build wsrep patched packages exclusively
+- OBS compatible
+
 * Mon Oct 06 2014 Balasubramanian Kandasamy <balasubramanian.kandasamy@oracle.com>
 - Add license info in each subpackage
 
@@ -1346,10 +1315,6 @@ echo "====="                                     >> $STATUS_HISTORY
 
 - Removal all traces of the readline library from mysql (BUG 13738013)
 
-* Wed Dec 07 2011 Alexey Yurchenko <alexey.yurchenko@codership.com>
-
-- wsrep-related cleanups.
-
 * Wed Sep 28 2011 Joerg Bruehe <joerg.bruehe@oracle.com>
 
 - Fix duplicate mentioning of "mysql_plugin" and its manual page,
@@ -1408,12 +1373,13 @@ echo "====="                                     >> $STATUS_HISTORY
 - Fix bug#12561297: Added the MySQL embedded binary
 
 * Thu Jul 07 2011 Joerg Bruehe <joerg.bruehe@oracle.com>
+
 - Fix bug#45415: "rpm upgrade recreates test database"
   Let the creation of the "test" database happen only during a new installation,
   not in an RPM upgrade.
   This affects both the "mkdir" and the call of "mysql_install_db".
 
-* Wed Feb 09 2011 Joerg Bruehe <joerg.bruehe@oracle.com>
+* Thu Feb 10 2011 Joerg Bruehe <joerg.bruehe@oracle.com>
 
 - Fix bug#56581: If an installation deviates from the default file locations
   ("datadir" and "pid-file"), the mechanism to detect a running server (on upgrade)
@@ -1534,7 +1500,7 @@ echo "====="                                     >> $STATUS_HISTORY
 - Fix some problems with the directives around "tcmalloc" (experimental),
   remove erroneous traces of the InnoDB plugin (that is 5.1 only).
 
-* Tue Oct 06 2009 Magnus Blaudd <mvensson@mysql.com>
+* Fri Oct 09 2009 Magnus Blaudd <mvensson@mysql.com>
 
 - Removed mysql_fix_privilege_tables
 
@@ -1731,7 +1697,7 @@ echo "====="                                     >> $STATUS_HISTORY
 
 - Set $LDFLAGS from $MYSQL_BUILD_LDFLAGS
 
-* Tue Mar 07 2006 Kent Boortz <kent@mysql.com>
+* Wed Mar 08 2006 Kent Boortz <kent@mysql.com>
 
 - Changed product name from "Community Edition" to "Community Server"
 
@@ -1826,7 +1792,7 @@ echo "====="                                     >> $STATUS_HISTORY
 
 * Thu Sep 29 2005 Lenz Grimmer <lenz@mysql.com>
 
-- fixed the removing of the RPM_BUILD_ROOT in the %clean section (the
+- fixed the removing of the RPM_BUILD_ROOT in the %%clean section (the
   $RBR variable did not get expanded, thus leaving old build roots behind)
 
 * Thu Aug 04 2005 Lenz Grimmer <lenz@mysql.com>
@@ -1979,7 +1945,7 @@ echo "====="                                     >> $STATUS_HISTORY
 
 - marked /etc/logrotate.d/mysql as a config file (BUG 2156)
 
-* Sat Dec 13 2003 Lenz Grimmer <lenz@mysql.com>
+* Fri Dec 12 2003 Lenz Grimmer <lenz@mysql.com>
 
 - fixed file permissions (BUG 1672)
 
@@ -2121,7 +2087,7 @@ echo "====="                                     >> $STATUS_HISTORY
 - Added separate libmysql_r directory; now both a threaded
   and non-threaded library is shipped.
 
-* Tue Sep 28 1999 David Axmark <davida@mysql.com>
+* Wed Sep 29 1999 David Axmark <davida@mysql.com>
 
 - Added the support-files/my-example.cnf to the docs directory.
 
